const moment = require('moment');
const { dateRange } = require('../Utils/Utility');
const { logger } = require('../logger');
const xlsx = require('xlsx');
const fs = require('fs');

const CONFIG = require('../Constants/Config');
const UserServices = require('../Services/CreateServices')("User");
const AttendanceServices = require('../Services/CreateServices')("Attendance");
const AttendanceSummaryServices = require('../Services/CreateServices')("AttendanceSummary");
const LeaveServices = require('../Services/CreateServices')("Leave");

// Haversine Formula to calculate distance
function haversineDistance(coords1, coords2) {
    const [lat1, lon1] = coords1;
    const [lat2, lon2] = coords2;
    const R = 6371; // Earth's radius in km

    const dLat = ((lat2 - lat1) * Math.PI) / 180;
    const dLon = ((lon2 - lon1) * Math.PI) / 180;

    const a =
        Math.sin(dLat / 2) ** 2 +
        Math.cos((lat1 * Math.PI) / 180) *
        Math.cos((lat2 * Math.PI) / 180) *
        Math.sin(dLon / 2) ** 2;

    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    return R * c * 1000; // Distance in km
}

const getUserCalendar = async (req, res) => {
    try {
        const { userId, month, year } = req.query;

        // Generate start and end dates for the month
        const startDate = moment(`${year}-${month}-01`).startOf("month").toDate();
        const endDate = moment(startDate).endOf("month").toDate();

        const calendar = await Calendar.find({
            userId,
            date: { $gte: startDate, $lte: endDate },
        }).sort({ date: 1 });

        res.status(200).json({
            success: true,
            data: calendar,
        });
    } catch (error) {
        console.error("Error fetching calendar:", error);
        res.status(500).json({ success: false, message: "Failed to fetch calendar" });
    }
};

module.exports.getDashboardCount = async (req, res) => {
    try {

        let responseCounts = {
            PENDING: { count: 0 },
            APPROVED: { count: 0 },
            REJECTED: { count: 0 }
        };

        const requestResponse = await AttendanceServices.aggregate([
            { $group: { _id: "$status", count: { $sum: 1 } } },
        ]);
        if (!requestResponse.status || requestResponse.error) return res.status(500).send(requestResponse);

        const statusCounts = requestResponse.data.map((st, id) => {
            responseCounts[st._id].count = st.count;
        });

        return res.status(200).send({ status: true, counts: responseCounts });
    }
    catch (e) {
        logger.error("🚀 ~ file: Payment.controller.js:48 ~ module.exports.getAllPayments= ~ e:" + e.toString())
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

module.exports.getAll = async (req, res) => {
    try {
        const user = req.local.user;
        const { status, logStatus, start, end, userId } = req.query;
        let query = {};

        if (["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1) {
            query = { userId: userId ? userId : user._id }
        } else {
            query = { userId: user._id }
        }

        if (logStatus) { query.logStatus = logStatus; }
        if (status) { query.status = status };

        query.date = dateRange(start, end)
        const startDate = query.date['$gte'];
        const endDate = query.date['$lte'];

        // Generate all dates in the range
        const allDates = [];
        let current = moment(startDate);
        while (current.isSameOrBefore(endDate, 'day') && current.isSameOrBefore(new Date(), 'day')) {
            allDates.push(current.format("YYYY-MM-DD"));
            current.add(1, 'day');
        }

        const attendanceRes = await AttendanceServices.aggregate([
            {
                $match: query
            },
            {
                $group: {
                    _id: {
                        userId: "$userId",
                        date: { $dateToString: { format: "%Y-%m-%d", date: "$date" } }
                    },
                    entries: {
                        $addToSet: {
                            checkIn: "$checkIn",
                            checkOut: "$checkOut",
                            workingHours: "$workingHours",
                            logStatus: "$logStatus",
                            attendanceType: '$attendanceType'
                        }
                    },
                    totalWorkingHours: {
                        $sum: "$workingHours"
                    },
                    checkInStatus: {
                        $max: {
                            $cond: [{ $ifNull: ["$checkIn.time", false] }, true, false]
                        }
                    }
                }
            },
            {
                $lookup: {
                    from: "users",
                    localField: "_id.userId",
                    foreignField: "_id",
                    as: "userDetails"
                }
            },
            {
                $unwind: "$userDetails"
            },
            {
                $project: {
                    _id: 0,
                    userId: "$_id.userId",
                    userName: { $concat: ["$userDetails.firstName", " ", "$userDetails.lastName"] },
                    date: "$_id.date",
                    entries: 1,
                    totalWorkingHours: 1,
                    checkInStatus: 1,
                    note: "",
                }
            },
            {
                $sort: { date: -1 }
            }
        ]);
        if (!attendanceRes.status || attendanceRes.error) return res.status(500).send(attendanceRes);

        // Merge attendance data with allDates
        const attendanceMap = {};
        attendanceRes.data.forEach(item => {
            attendanceMap[item.date] = { ...item, isAllowedAddLogs: true };
        });

        const completeAttendance = allDates.reverse().map(date => {
            return attendanceMap[date] || {
                userId: query.userId,
                userName: "",
                date: date,
                entries: [],
                totalWorkingHours: 0,
                checkInStatus: false,
                isAllowedAddLogs: true,
                note: "",
            };
        });

        return res.status(200).send({ status: true, query, data: completeAttendance });
    } catch (e) {
        logger.error("🚀 ~ file: Attendance.controller.js:11 ~ module.exports.getAll= ~ e:" + JSON.stringify(e));
        return res.status(200).send({ status: false, error: e.toString() });
    }
}

module.exports.get = async (req, res) => {
    try {
        const id = req.params.id;

        const attendanceRes = await AttendanceServices.find({ query: { _id: id }, populate: ["userId"] });
        if (!attendanceRes.status || attendanceRes.error) return res.status(500).send(attendanceRes);
        if (attendanceRes.data.length == 0) return res.status(404).send({ status: false, error: "No Ticket Found!" });

        const dayAttendanceRes = await AttendanceServices.find({ query: { userId: attendanceRes.data[0].userId, date: attendanceRes.data[0].date, _id: { $ne: attendanceRes.data[0]._id } }, populate: ["userId"] });

        return res.status(200).send({ status: true, data: attendanceRes.data[0], dayAttendanceLog: dayAttendanceRes?.data || [] });
    } catch (e) {
        logger.error("🚀 ~ file: Attendance.controller.js:25 ~ module.exports.createTicket= ~ e:" + JSON.stringify(e));
        return res.status(200).send({ status: false, error: e.toString() });
    }
}

module.exports.status = async (req, res) => {
    try {
        const user = req.local.user;
        const userId = req.local.user._id.toString();

        const date = moment().startOf('day').toDate(); // Ensure the date is the start of the day
        // Check if an open check-in already exists
        const todaysAttendance = await AttendanceServices.Model.find({
            userId: userId,
            date,
            orgId: user.organization._id,
            "checkOut.time": { $ne: null },
        });
        const todaysWorkingHours = todaysAttendance.reduce((prev, val, id) => { return prev + +val.workingHours }, 0)
        console.log("=> hours: ", todaysAttendance, todaysWorkingHours)
        const openAttendance = await AttendanceServices.Model.findOne({
            userId: userId,
            date,
            orgId: user.organization._id,
            "checkOut.time": null,
        });

        if (openAttendance) {
            return res.status(200).send({
                status: true,
                data: {
                    todaysWorkingHours,
                    attendanceData: openAttendance,
                    checkInTime: moment(openAttendance.checkIn.time).format('DD-MM-YYYY hh:mm'),
                    checkedIn: true,
                    shiftStartTime: CONFIG.ATTENDANCE.START,
                    shiftEndTime: CONFIG.ATTENDANCE.END,
                    attendance: {
                        present: 0,
                        absent: 0,
                    },
                    leave: {
                        total: 0,
                        available: 0,
                        taken: 0
                    },
                },
                message: `Checked In at ${moment(openAttendance.checkIn.time).local().format('DD-MM-YYYY hh:mm:ss')}`
            });
        }

        return res.status(200).send({
            status: true,
            data: {
                todaysWorkingHours,
                checkedIn: false,
                shiftStartTime: CONFIG.ATTENDANCE.START,
                shiftEndTime: CONFIG.ATTENDANCE.END,
                attendance: {
                    present: 0,
                    absent: 0,
                },
                leave: {
                    total: 0,
                    available: 0,
                    taken: 0
                }
            }
        });
    } catch (e) {
        logger.error("Error in attendanceController.js status: " + JSON.stringify(e));
        return res.status(500).send({ status: false, error: e.toString() });
    }
}

module.exports.checkin = async (req, res) => {
    try {
        const user = req.local.user;
        const { latitude, longitude } = req.body;

        const date = moment().startOf('day').toDate(); // Ensure the date is the start of the day
        const checkInTime = new Date();

        // Check if an open check-in already exists
        const openAttendance = await AttendanceServices.Model.findOne({
            userId: user._id,
            date,
            "checkOut.time": null,
        });

        if (openAttendance) {
            return res.status(400).send({ status: false, message: "Already checked in and not checked out yet." });
        }

        // Create a new check-in record
        const attendanceData = {
            userId: user._id,
            orgId: user.organization._id,
            date,
            checkIn: {
                time: checkInTime,
                geo: { type: 'Point', coordinates: [longitude || 0, latitude || 0] }
            },
            present: true,
        };

        const result = await AttendanceServices.create(attendanceData);
        if (!result.status || result.error) {
            return res.status(500).send(result);
        }

        return res.status(200).send({ status: true, message: "Check-in recorded successfully.", data: result.data });
    } catch (e) {
        logger.error("Error in AttendanceController checkin: " + JSON.stringify(e));
        return res.status(500).send({ status: false, error: e.toString() });
    }
}

module.exports.checkout = async (req, res) => {
    const user = req.local.user;
    const orgId = user.organization._id;
    const { longitude, latitude } = req.body;
    const date = moment().startOf('day').toDate(); // Ensure the date is the start of the day

    try {
        // Find attendance record for the user for today
        const attendance = await AttendanceServices.Model.findOne({ userId: user._id, orgId, date, "checkOut.time": null, });

        if (!attendance || !attendance.checkIn.time) {
            return res.status(400).json({ message: 'No check-in record found for today' });
        }

        // Set checkout details
        const checkoutTime = new Date();
        attendance.checkOut = {
            time: checkoutTime,
            geo: { type: 'Point', coordinates: [longitude || 0, latitude || 0] }
        };

        // Calculate working hours (in hours with decimals)
        const checkInTime = moment(attendance.checkIn.time);
        const duration = moment.duration(moment(checkoutTime).diff(checkInTime));
        const workingHours = duration.asHours();

        attendance.workingHours = workingHours;
        attendance.present = true; // Mark present since the user checked out
        attendance.status = true; // Optional: Mark status as complete for the day

        await attendance.save();

        return res.status(200).json({
            status: true,
            message: 'Check-out recorded successfully',
            workingHours,
            attendance,
        });
    } catch (error) {
        logger.error("AttendanceController checkout error: " + JSON.stringify(error));
        return res.status(500).json({ status: false, error: error.message });
    }
};

module.exports.manualEntryLog = async (req, res) => {
    const user = req.local.user;
    const orgId = user.organization._id;
    let { start, end, logStatus, searchText } = req.query;
    let { startIndex, limit } = req.local;

    try {

        let query = {
            attendanceType: "MANUAL_ENTRY",
            logStatus: logStatus || "PENDING"
        };

        if (!!start && !!end) { query.date = dateRange(start, end) }

        const result = await AttendanceServices.find({ query, populate: ["userId"], start: startIndex, limit: limit, sort: { date: -1 } });
        if (!result.status || result.error) {
            return res.status(500).send(result);
        }

        return res.status(200).send({ status: true, data: result.data, count: result.count });
    } catch (error) {
        logger.error("AttendanceController manualEntryLog Error: " + JSON.stringify(error));
        return res.status(500).json({ status: false, error: error.message });
    }
};

module.exports.manualEntryLogCreate = async (req, res) => {
    const user = req.local.user;
    const orgId = user.organization._id;
    let { checkInTime, checkOutTime, date: bodyDate, longitude, latitude } = req.body;
    const date = moment(bodyDate || checkInTime).startOf('day').toDate(); // Ensure the date is the start of the day

    try {
        // Calculate working hours (in hours with decimals)
        checkInTime = moment(checkInTime);
        const duration = moment.duration(moment(checkOutTime).diff(checkInTime));
        const workingHours = duration.asHours();

        // Create a new check-in record
        const attendanceData = {
            userId: user._id,
            orgId: user.organization._id,
            date,
            checkIn: {
                time: checkInTime,
                geo: { type: 'Point', coordinates: [longitude || 0, latitude || 0] }
            },
            checkOut: {
                time: checkOutTime,
                geo: { type: 'Point', coordinates: [longitude || 0, latitude || 0] }
            },
            attendanceType: "MANUAL_ENTRY",
            present: true,
            workingHours,
            logStatus: "PENDING"
        };

        const result = await AttendanceServices.create(attendanceData);
        if (!result.status || result.error) {
            return res.status(500).send(result);
        }

        return res.status(200).send({ status: true, message: "Manual Entry log recorded successfully.", data: result.data });
    } catch (error) {
        logger.error("AttendanceController manualEntryLogCreate error: " + JSON.stringify(error));
        return res.status(500).json({ status: false, error: error.message });
    }
};

module.exports.manualEntryLogUpdate = async (req, res) => {
    try {
        const user = req.local.user;
        const orgId = user.organization._id;
        const id = req.params.id;
        const { checkInTime, checkOutTime, date, logStatus } = req.local.log;
        let updateData = {};

        const attendanceRes = await AttendanceServices.findById(id);
        if (!attendanceRes.status || attendanceRes.error) {
            return res.status(500).send(attendanceRes);
        }
        const hasAccess = ["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1

        if (!hasAccess) {
            return res.status(401).send({ status: false, error: 'No access! Contact administrator.' })
        }
        if (moment(checkInTime).isAfter(checkOutTime)) {
            return res.status(409).send({ status: false, error: "Checkout Time must be less then checkin" })
        }

        if (checkInTime) { updateData = { ...updateData, "checkIn": { time: checkInTime } } }
        if (checkOutTime) { updateData = { ...updateData, "checkOut": { time: checkOutTime } } }
        if (date) { updateData = { ...updateData, date } }
        if (logStatus) {
            if(["APPROVED", "REJECTED", "COMPLETED"].indexOf(logStatus) != -1){
                updateData = { ...updateData, approvedBy: user, approvedAt: new Date() }
            }
            updateData = { ...updateData, logStatus }
        }

        const result = await AttendanceServices.updateById(id, updateData)
        return res.status(200).send({ status: true, message: "Manual Entry Log Updated successfully.", data: result.data });
    } catch (error) {
        logger.error("AttendanceController manualEntryLogUpdate error: " + JSON.stringify(error));
        return res.status(500).json({ status: false, error: error.message });
    }
};

module.exports.manualEntryLogDelete = async (req, res) => {
    const user = req.local.user;
    const id = req.params.id;

    try {
        const attendanceRes = await AttendanceServices.findById(id);
        if (!attendanceRes.status || attendanceRes.error) {
            return res.status(500).send(attendanceRes);
        }
        const hasAccess = ["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1

        if (!hasAccess) {
            return res.status(401).send({ status: false, error: 'No access! Contact administrator.' })
        }

        const result = await AttendanceServices.deleteById(id);

        return res.status(200).send({ status: true, message: "Manual Entry Log Deleted." });
    } catch (error) {
        logger.error("AttendanceController manualEntryLogDelete" + JSON.stringify(errror));
        return res.status(500).json({ status: false, error: error.message });
    }
};

// Export Attendance API
module.exports.reportOld = async (req, res) => {
    try {
        const { startDate, endDate, userId, download } = req.query;

        const currentDate = Date.now();
        const start = startDate ? new Date(startDate) : new Date(moment(currentDate).startOf('month'));
        const end = endDate ? new Date(endDate) : new Date(moment(currentDate).endOf('month'));

        const filter = {
            date: dateRange(start, end) //{ $gte: start, $lte: end },
        };
        if (userId) filter.userId = userId;

        // Fetch attendance data from MongoDB
        const attendanceData = await AttendanceServices.Model.find(filter).populate('userId'); // Assuming userId is populated with name.

        // Generate a list of all dates in the range
        const allDates = [];
        for (let d = new Date(start); d <= end; d.setDate(d.getDate() + 1)) {
            allDates.push(d.toISOString().split('T')[0]);
        }

        // Group and combine data by userId and date
        const userAttendance = {};
        attendanceData.forEach((entry) => {
            const userId = entry.userId._id.toString();
            const userObj = entry.userId;
            const dateKey = entry.date.toISOString().split('T')[0];

            if (!userAttendance[userId]) userAttendance[userId] = { userObj: userObj, 'dates': [] };
            if (!userAttendance[userId]['dates'][dateKey]) {
                userAttendance[userId]['dates'][dateKey] = {
                    date: dateKey,
                    checkIn: entry.checkIn,
                    checkOut: entry.checkOut,
                    workingHours: entry.workingHours || 0,
                };
            } else {
                const existing = userAttendance[userId]['dates'][dateKey];
                existing.workingHours += entry.workingHours || 0;

                // Update check-in time to the earliest
                if (entry.checkIn.time < existing.checkIn.time) {
                    existing.checkIn = entry.checkIn;
                }

                // Update check-out time to the latest
                if (entry.checkOut.time > existing.checkOut.time) {
                    existing.checkOut = entry.checkOut;
                }
            }
        });

        // Prepare Excel data
        const workbook = {};
        const attendanceSummary = {};

        for (const [userId, { userObj, dates }] of Object.entries(userAttendance)) {
            // Add present/absent calculations
            const loggedDates = Object.keys(dates);
            const presentCount = loggedDates.length;
            const absentCount = allDates.filter((d) => !loggedDates.includes(d)).length;

            const rows = [
                [`Start Date:`, `${moment(start).format('DD-MM-YYYY')}`, `End Date:`, `${moment(end).format('DD-MM-YYYY')}`],
                [`Present Days:`, `${presentCount}`, `Absent Days:`, `${absentCount}`],
                [],
                [],
                ['Date', 'Check-In Time', 'Check-Out Time', 'Working Hours', 'Check-In Location', 'Check-Out Location', 'Distance (m)'],
            ];

            for (const record of Object.values(dates)) {
                const checkInCoords = record.checkIn.geo?.coordinates || [0, 0];
                const checkOutCoords = record.checkOut.geo?.coordinates || [0, 0];
                const distance = haversineDistance(checkInCoords, checkOutCoords);

                rows.push([
                    record.date,
                    record.checkIn.time ? record.checkIn.time.toISOString() : 'N/A',
                    record.checkOut.time ? record.checkOut.time.toISOString() : 'N/A',
                    record.workingHours.toFixed(2),
                    `https://maps.google.com/?q=${checkInCoords[1]},${checkInCoords[0]}`,
                    `https://maps.google.com/?q=${checkOutCoords[1]},${checkOutCoords[0]}`,
                    distance.toFixed(2)
                ]);
            }

            // Add to summary
            attendanceSummary[userId] = {
                firstName: userObj.firstName,
                lastName: userObj.lastName,
                present: presentCount,
                absent: absentCount,
                data: rows
            };

            // Add the sheet to the workbook
            workbook[`${userObj.firstName} ${userObj.lastName}`] = rows;
        }

        if (download === 'true') {
            // Create Excel file
            const sheets = Object.entries(workbook).map(([sheetName, data]) => ({
                name: sheetName,
                data,
            }));

            const filePath = `attendance_${Date.now()}.xlsx`;
            const wb = xlsx.utils.book_new();

            sheets.forEach(({ name, data }) => {
                const ws = xlsx.utils.aoa_to_sheet(data);
                xlsx.utils.book_append_sheet(wb, ws, name);
            });

            xlsx.writeFile(wb, filePath);

            // Send file to client
            return res.download(filePath, 'Attendance_Report.xlsx', (err) => {
                if (err) {
                    console.error(err);
                }
                fs.unlinkSync(filePath); // Clean up the file after sending
            });
        } else {
            // Send summary in JSON format
            return res.json({ summary: attendanceSummary });
        }
    } catch (error) {
        logger.error('Error exporting attendance:' + JSON.stringify(error));
        return res.status(500).send({ status: false, error: 'Failed to export attendance.' });
    }
};

module.exports.report = async (req, res) => {
    try {
        const { startDate, endDate, userId, download } = req.query;

        // Set default start and end dates if not provided
        const currentDate = Date.now();
        const start = startDate ? new Date(startDate) : new Date(moment(currentDate).startOf('month'));
        const end = endDate ? new Date(endDate) : new Date(moment(currentDate).endOf('month'));

        const holidays = [
            '2023-10-02', // Example holiday
            '2023-12-25', // Example holiday
        ];

        // Fetch attendance data from MongoDB
        const filter = {
            date: { $gte: start, $lte: end },
        };
        if (userId) filter.userId = userId;

        const attendanceData = await AttendanceServices.Model.find(filter).populate('userId');

        // Fetch leave records for the date range
        const leaveRecords = await Leave.find({
            from: { $lte: end }, // Leave starts on or before the end date
            to: { $gte: start }, // Leave ends on or after the start date
            leaveStatus: 'APPROVED', // Only consider approved leaves
        }).populate('userId');

        // Group attendance data by user
        const userAttendance = {};
        attendanceData.forEach((entry) => {
            const userId = entry.userId._id.toString();
            const dateKey = moment(entry.date).format('YYYY-MM-DD');

            if (!userAttendance[userId]) {
                userAttendance[userId] = {
                    userObj: entry.userId,
                    dates: {},
                };
            }

            userAttendance[userId].dates[dateKey] = {
                checkIn: entry.checkIn,
                checkOut: entry.checkOut,
                workingHours: entry.workingHours || 0,
            };
        });

        // Prepare Excel data
        const sheetData = [];

        // Loop through each user and generate their attendance row
        for (const [userId, { userObj, dates }] of Object.entries(userAttendance)) {
            let present = 0, absent = 0, weekend = 0, workingHours = 0, leaves = 0
            // Add user details
            sheetData.push([
                "Person ID", userId, "",
                "Employee Name", "", `${userObj.firstName} ${userObj.lastName}`, "", "",
            ]);

            // Add date headers
            const dateRow = ["Date"];
            const checkInRow = ["Check-in"];
            const checkOutRow = ["Check-out"];
            const workingHourRow = ["Attanded"];
            const statusRow = ["Status"];

            allDates.forEach((date) => {
                dateRow.push(moment(date).format('DD'));
                workingHourRow = dates[date]?.workingHours || 0;
                if (dates[date] && dates[date].checkIn.time) {
                    checkInRow.push(moment(dates[date].checkIn.time).format('HH:mm'));
                } else if (holidays.includes(date)) {
                    checkInRow.push("#");
                } else {
                    checkInRow.push("-");
                }

                if (dates[date] && dates[date].checkOut.time) {
                    checkOutRow.push(moment(dates[date].checkOut.time).format('HH:mm'));
                } else if (holidays.includes(date)) {
                    checkOutRow.push("#");
                } else {
                    checkOutRow.push("-");
                }

                if (holidays.includes(date)) {
                    statusRow.push("#");
                } else if (dates[date] && dates[date].checkIn.time) {
                    statusRow.push("P");
                } else {
                    statusRow.push("A");
                }
            });

            sheetData.push(dateRow, checkInRow, checkOutRow);
            // Add OT, Late, Early Leave, Attended, Break, and Status rows
            sheetData.push(["OT", ...Array(allDates.length).fill("0")]);
            sheetData.push(["Late", ...Array(allDates.length).fill("0")]);
            sheetData.push(["Early Leave", ...Array(allDates.length).fill("0")]);
            sheetData.push(workingHourRow);
            sheetData.push(["Break", ...Array(allDates.length).fill("0")]);
            sheetData.push(statusRow);

            // Add summary row
            sheetData.push([
                "Summary",
                `Normal Attendance:${present}`, `Weekend:${weekend}`, `Leave:${leaves}`, `Overtime Duration:${0}`,
                `Overtime:${0}`, `Attended Duration:${workingHours}`, `Absence:${absent}`,
                `Late:${0}`, `Early-Leave:${0}`, `Break Duration:${0}`,
                ...Array(allDates.length - 1).fill("")
            ]);

            // Add an empty row between users
            sheetData.push([], []);
        }

        // Create Excel sheet
        const workbook = xlsx.utils.book_new();
        const sheet = xlsx.utils.aoa_to_sheet(sheetData);
        xlsx.utils.book_append_sheet(workbook, sheet, "Attendance Report");

        if (download === 'true') {
            // Save and send the file
            const filePath = `attendance_report_${Date.now()}.xlsx`;
            xlsx.writeFile(workbook, filePath);

            return res.download(filePath, 'Attendance_Report.xlsx', (err) => {
                if (err) {
                    console.error(err);
                }
                fs.unlinkSync(filePath); // Clean up the file after sending
            });
        } else {
            // Send JSON response
            return res.json({ data: sheetData });
        }
    } catch (error) {
        logger.error('Error generating report: ' + JSON.stringify(e));
        return res.status(500).send({ status: false, error: 'Failed to generate report.' });
    }
};