const fs = require('fs');
const path = require('path');

const ClientServices = require('../Services/CreateServices')('Clients');
const BranchServices = require('../Services/Branch.services');
const OrganizationServices = require('../Services/Organization.services');
const ZoneServices = require('../Services/Zone.services');
const { logger } = require('../logger');
const utilitiesController = require('../Controllers/Utilities.controller');

/**
 * @description This function is used for get all clients i.e. Banks
 * @param {*} req 
 * @param {*} res 
 * @returns Object ClientResult {status : true, clients : array}
 * - { status: false, error: string }
 */
module.exports.getAllClients = async (req, res) => {
    try {
        var count = 0;
        let { type, searchValue, withCount = "false" } = req.query;

        let query = { /*'organization._id': req.local.user.organization._id; */ };
        withCount = (withCount == "true") ? true : false;
        if (type) { query.type = { $in: type.split(',').map((t, id) => t.trim()) } }
        if (searchValue) {
            query['$or'] = [
                { type: { $regex: searchValue, $options: 'i' } },
                { name: { $regex: searchValue, $options: 'i' } },
                { initial: { $regex: searchValue, $options: 'i' } }
            ]
        }
        const ClientResultCount = await ClientServices.count(query);
        if (!ClientResultCount.status || ClientResultCount.error) return res.status(500).send(ClientResultCount);

        const ClientResult = await ClientServices.aggregate([
            { $match: query },
            { $sort: { 'name': 1 } },
            { $skip: req.local.startIndex },
            { $limit: req.local.limit },
            ...(withCount ? [
                {
                    $addFields: {
                        id: { $toString: "$_id" }
                    }
                },
                {
                    $lookup: {
                        from: "cases",
                        let: { clientId: "$id", },
                        pipeline: [
                            {
                                $match: { $expr: { $eq: ["$$clientId", "$client._id"] } },
                            },
                            {
                                $project: { _id: 1, caseNumber: 1, type: 1 },
                            }
                        ],
                        as: "cases",
                    }
                },
                {
                    $addFields: {
                        cases: {
                            $size: {
                                $filter: {
                                    input: "$cases",
                                    as: "case",
                                    cond: { $and: [{ $eq: ["$$case.type", "CASE"] }] },
                                },
                            },
                        },
                        leads: {
                            $size: {
                                $filter: {
                                    input: "$cases",
                                    as: "case",
                                    cond: { $and: [{ $ne: ["$$case.type", "CASE"] }] },
                                },
                            },
                        },
                    }
                }
            ] : []),
        ]);
        if (!ClientResult.status || ClientResult.error) return res.status(500).send(ClientResult);

        count = ClientResultCount.count;

        return res.status(200).send({ status: true, clients: ClientResult.data, count: count });
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.getAllClients= ~ e:` + e.toString())
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This function is used for get only one client from requested Id of client.
 * @param {String} req.params from that get Client Id.  
 * @param {*} res 
 * @returns Object ClientResult {status : true, clients : Object}
 * - { status: false, error: string }
 */
module.exports.getClient = async (req, res) => {
    try {
        let id = req.params.id;

        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const ClientResult = await ClientServices.findById(id);
        if (!ClientResult.status || ClientResult.error) return res.status(500).send(ClientResult);

        return res.status(200).send({ status: ClientResult.status, clients: ClientResult.data });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for Create Client
 * @param {*} req 
 * @param {*} res 
 * @returns Object client_res { status: true, clients: Object }
 * - { status: false, error: string }
 */
module.exports.createClient = async (req, res) => {
    try {
        var data = req.local.client;

        const alreadyCreated = await ClientServices.find({ query: { name: data.name } })
        if (!alreadyCreated.status || alreadyCreated.error) return res.status(500).send(alreadyCreated);

        if (alreadyCreated.count == 0) {

            // get Login UserId
            data.createdBy = req.local.user._id;

            data.uniqueId = Math.floor(100000 + Math.random() * 900000);

            const organization = req.local.user.organization;

            const Organization = await OrganizationServices.getById(organization);
            if (!Organization.status) return res.status(500).send(Organization);

            data.organization = Organization.Organizations;

            const client_res = await ClientServices.create(data);
            if (!client_res.status) return res.status(500).send(client_res);

            if (req.file) {
                const cid = client_res.data._id;
                data.logo = req.file.path;

                if (!fs.existsSync(process.env.CDN_PATH + '/client/' + cid)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/client/' + cid, { recursive: true });
                }
                const fileName = "logo_" + req.file.filename;

                let currentPath = process.cwd() + "/" + req.file.path;
                const destination = path.join(process.env.CDN_PATH, "client", cid, fileName);

                // call function for move file
                await utilitiesController.moveFile(currentPath, destination);

                const filepath = path.join("client/", cid, fileName);
                data.logo = filepath;
            }


            return res.status(200).send({ status: client_res.status, clients: client_res.data });
        }
        else {
            return res.status(500).send({ status: false, error: "Already Created!" });
        }

    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.createClient= ~ err:` + err.toString())
        return res.status(500).send({ status: false, error: err.toString() });
    }
};
/**
 * @description This function is used for update Client
 * @param {*} req 
 * @param {*} res 
 * @returns Object client_res { status: true, clients: Object }
 * - { status: false, error: string }
 */
module.exports.updateClient = async (req, res) => {
    try {
        const id = req.params.id;
        var data = req.local.client;

        data.updatedBy = req.local.user._id;

        if (req.file) {
            data.logo = req.file.path;

            if (!fs.existsSync(process.env.CDN_PATH + '/client/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/client/' + id, { recursive: true });
            }

            const fileName = "logo_" + req.file.filename;
            let currentPath = process.cwd() + "/" + req.file.path;
            const destination = path.join(process.env.CDN_PATH, "client", id, fileName);
            // call function for move file
            await utilitiesController.moveFile(currentPath, destination);

            const filepath = path.join("client", id, fileName);
            data.logo = filepath;

        }

        const client_res = await ClientServices.updateById(id, data);
        if (!client_res.status) return res.status(500).send(client_res);

        return res.status(200).send({ status: client_res.status, clients: client_res.data });

    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.updateClient= ~ err:` + err.toString());
        return res.status(500).send({ status: false, error: err.toString() });
    }
};

/**
 * @description This function is used for get all branches from bank/clientId
 * @param {*} req 
 * @param {*} res 
 * @returns Object clientResult { status: true, branches: array }
 * - { status: false, error: string }
 */
module.exports.getClientBranches = async (req, res) => {
    try {
        const id = req.params.id;
        const { type, searchText } = req.query;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        let query = { 'client._id': id };

        if (type) { query.type = type };
        if (searchText) {
            query['$and'] = [];
            query['$and'].push({
                '$or': [
                    { "name": { $regex: searchText, $options: 'i' } },
                    { "address": { $regex: searchText, $options: 'i' } },
                    { "ledgerName": { $regex: searchText, $options: 'i' } },
                ]
            });
        }

        const clientResult = await BranchServices.getAllBranches(query);
        if (!clientResult.status || clientResult.error) return res.status(500).send(clientResult);

        return res.status(200).send(clientResult);
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.getClientBranches= ~ e:` + e.toString())
        return res.status(500).send({ status: false, error: e.toString() });
    }
};
/**
 * @description This function is used for get zoneList from clientId
 * @param {*} req 
 * @param {*} res 
 * @returns Object clientResult { status: true, zones: array }
 * - { status: false, error: string }
 */
module.exports.getClientZones = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const clientResult = await ZoneServices.getAllZones({ "client._id": id });
        if (!clientResult.status || clientResult.error) return res.status(500).send(clientResult);

        return res.status(200).send(clientResult);

    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.getClientZones= ~ e:` + e.toString())
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

module.exports.deleteClient = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};

