const fs = require('fs');
const path = require('path');
const LeadServices = require('../Services/Lead.services');
const ClientServices = require('../Services/Client.services');
const ZoneServices = require('../Services/Zone.services');
const BranchServices = require('../Services/Branch.services');
const caseServices = require('../Services/Case.services');
const UserServices = require('../Services/User.services');
const OrganizationServices = require('../Services/Organization.services');
const TaskServices = require('../Services/Task.services');
const nInvoiceServices = require('../Services/CreateServices')('nInvoice');
const CreditOfficersServices = require('../Services/CreateServices')('CreditOfficer');
const nInvoiceController = require('../Controllers/nInvoice.controller');

const caseController = require('../Controllers/Case.controller');
const utilitiesController = require('../Controllers/Utilities.controller');
const TaskController = require('../Controllers/Task.controller');
const { workType } = require('../Types/Task');
const moment = require('moment');
const base64toFile = require('../Utils/Base64ToFile');
const { logger } = require('../logger');

/**
 * @description This is used for get list of all Leads from the Database
 * @param {*} req 
 * @param {*} res 
 * @returns Object LeadResult { status: true, Leads: Array } 
 * - { status : false, error : string }
 */
module.exports.getAllLeads = async (req, res) => {
    try {
        let query = {};
        const user = req.local.user;
        const { pickupPerson, searchValue, client, branch, createdBy, caseStatus, leadFrom, typeOfReport, fullAddress, type } = req.query
        // query['organization._id'] = req.local.user.organization._id;
        // query['clientType.label'] = { $eq: "BANK" };

        query['type'] = !!type ? { $in: [...type.split(',')] } : "LEAD";

        if (pickupPerson) { query['createdBy'] = pickupPerson; }
        if (client) { query['client._id'] = client; }
        if (branch) { query['branch._id'] = branch; }
        if ((!['VALUER', 'ADMIN', 'FRONTDESK'].includes(user.role)) && createdBy) { query['createdBy'] = createdBy; }

        if (caseStatus) { query['caseStatus'] = { $eq: caseStatus } }
        else { query['caseStatus'] = { $ne: 'CANCELLED' } }

        if (leadFrom) { query["leadFrom.label"] = { $regex: leadFrom, $options: 'i' } }
        if (typeOfReport) { query["typeOfReport.label"] = { $regex: typeOfReport, $options: 'i' } }

        if (searchValue) {
            query['$or'] = [
                { fullAddress: { $regex: searchValue, $options: 'i' } },
                { caseNumber: { $regex: searchValue, $options: 'i' } },
                { ownerName: { $regex: searchValue, $options: 'i' } },
                { "leadFrom.label": { $regex: searchValue, $options: 'i' } },
                { "client.name": { $regex: searchValue, $options: 'i' } },
                { "client.initial": { $regex: searchValue, $options: 'i' } },
                { "branch.name": { $regex: searchValue, $options: 'i' } },
                { "typeOfReport.label": { $regex: searchValue, $options: 'i' } },
                { "address": { $regex: searchValue, $options: 'i' } }
            ]
        }

        const LeadCountResult = await LeadServices.getCount(query);
        if (!LeadCountResult.status || LeadCountResult.error) return res.status(500).send(LeadCountResult);
        let count = LeadCountResult.Leads || 0;

        const LeadResult = await LeadServices.getAllLeads(query, req.local.startIndex, req.local.limit);
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        return res.status(200).send({ status: true, Leads: LeadResult.Leads, count: count });
    }
    catch (e) {
        console.log("🚀 ~ file: Lead.controller.js:77 ~ module.exports.getAllLeads= ~ e:", e)
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Lead 
 * @param {*} req.params.id  from that get LeadID
 * @param {*} res
 * @returns Object LeadResult  { status: true, Leads: object } 
 * - { status : false, error : string }
 */
module.exports.getLead = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const LeadResult = await LeadServices.getLead(id);
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        const tasks = await TaskServices.findAll({ caseId: id }, {});

        const hasInvoiceRes = await nInvoiceServices.find({ query: { 'cases._id': id, invoiceStatus: { $in: ["OPEN", "COMPLETED"] } } });
        if (!hasInvoiceRes.status || hasInvoiceRes.error) return res.status(500).send(hasInvoiceRes);

        return res.status(200).send({ ...LeadResult, tasks: tasks, invoice: hasInvoiceRes.data, });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for create new lead 
 * @param {*} req 
 * @param {*} res 
 * @returns  Object LeadResult { status: true, Leads: Object }
 * -  { status: false, error: string }
 */
module.exports.createLead = async (req, res) => {
    try {
        const datas = req.local.lead;
        const orgId = req.local.lead.organizationId || req.local.user.organization;
        const leadType = (req.local.lead.leadType && req.local.lead.leadType.value) ? req.local.lead.leadType.value : "LEAD";
        datas.type = leadType;

        if (datas.uniqueNo) {
            const hasLeadResult = await LeadServices.getAllLeads({ uniqueNo: datas.uniqueNo }, 0, 10);
            if (!hasLeadResult.status || hasLeadResult.error || hasLeadResult.Leads.length > 0) return res.status(500).send({ status: false, error: "Lead Already Created." });
        }

        const Organizations = await OrganizationServices.getById(orgId);
        if (!Organizations.status || Organizations.error) return res.status(500).send(Organizations);
        datas.organization = Organizations.Organizations;

        if (['INDIVIDUAL', 'PRIVATE_CLIENT'].includes(datas.clientType.value)) {
            const hasClient = await ClientServices.getAllClients({ name: datas.ownerName })
            if (!hasClient.status || hasClient.error) { return res.status(500).send(hasClient) };
            let data = { type: datas.clientType.value };

            if (hasClient.clients.length == 0) {
                data.name = datas.ownerName;
                data.phone = datas.ownerPhoneNo;
                data.displayName = datas.ownerName;
                data.initial = datas.ownerName.slice(0, 1);
                data.ledgerName = datas.ownerName;
                data.address = datas.fullAddress;
                data.createdBy = req.local.user._id;
                data.uniqueId = Math.floor(100000 + Math.random() * 900000);
                data.email = data.uniqueId + "_owner@gmail.com";

                data.organization = Organizations.Organizations;

                const client_res = await ClientServices.createClient(data);
                if (!client_res.status) return res.status(500).send(client_res);

                datas.client = client_res.clients;
            }
            else {
                datas.client = hasClient.clients[0];
            }
        }
        else {
            const creditOfficer = await CreditOfficersServices.findById(datas.creditOfficer);
            if (!creditOfficer.status || creditOfficer.error) return res.status(500).send(creditOfficer);
            datas.creditOfficer = creditOfficer.data;

            const Client = await ClientServices.getClientById(datas.client);
            if (!Client.status || Client.error) return res.status(500).send(Client);
            datas.client = Client.clients;

            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status || Branch.error) return res.status(500).send(Branch);
            datas.branch = Branch.branches;

            if (datas.zone) {
                const Zone = await ZoneServices.getZone(datas.zone);
                if (!Zone.status || Zone.error) return res.status(500).send(Zone);
                datas.zone = Zone.zones;
            }
        }

        if (datas.owners) {
            datas.owners.map((owner) => {
                const owner_id = Math.floor(100000 + Math.random() * 900000);
                owner.id = owner_id;
            })
        }

        if (datas.documents) {
            datas.documents.map((document) => {
                // console.log(document);
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                document.id = doc_id;
                document.uploadedBy = req.local.user._id;
                document.createdAt = new Date();
            })
        }

        const case_result = await caseServices.find({ limit: 1 });
        if (!case_result.status || case_result.error) return res.status(500).send(case_result);
        // console.log(case_result.cases);

        if (case_result.cases.length == 0) {
            const str = "" + 1, pad = "000000"
            const answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }
        else {
            var caseNo = case_result.cases[0].caseNumber;
            var caseNumber = parseInt(caseNo) + 1;
            const str = "" + caseNumber, pad = "000000"
            var answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }

        datas.createdBy = req.local.user._id;
        let caseHistory = [];
        caseHistory = [{
            "id": Math.floor(100000 + Math.random() * 900000),
            "title": "Create Lead",
            "description": "Create Lead",
            "datetime": new Date(),
        }];

        datas.caseHistory = caseHistory;
        datas.caseStatus = "LEAD";

        const LeadResult = await LeadServices.createLead(datas);
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        // create TASK
        const pickedUpTaskRes = await TaskController.createTask({ organizationId: Organizations.Organizations._id, caseId: LeadResult.Leads._id, workType: workType.LEAD, users: [{ ...req.local.user.toJSON(), userId: req.local.user._id }], status: 'COMPLETED', completedAt: new Date() });

        if (datas.isAdvanceInvoice) {
            // Create Task for advance invoice if true
            const advanceInvoiceTaskRes = await TaskController.createTask({ organizationId: Organizations.Organizations._id, caseId: LeadResult.Leads._id, workType: workType.PREPARE_INVOICE });

            // Create Invoice
            const advanceInvoiceRes = await nInvoiceServices.create({
                organization: Organizations.Organizations,
                cases: [{...LeadResult.Leads.toJSON(), fees: LeadResult.Leads.toJSON().fees.expectedFees || 0 }],
                invoiceType: "PROFORMA",
                isAdvance: true,
                amount: datas.invoiceAmount || 0,
                requestedBy: req.local.user,
                invoiceStatus: 'OPEN'
            })
        }

        // From here call createCaseLog function in case controller for create caseLog
        req.body.caseStatus = "CREATELEAD";
        const createCaseLog = await caseController.createCaseLog(req);

        // ========================== Document, Image ==========================
        const id = LeadResult.Leads._id.toString();
        if (LeadResult.Leads.documents.length > 0) {

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
            }

            await Promise.all(LeadResult.Leads.documents.map(async file => {

                const fileName = "document_" + file.fileName;
                let currentPath = process.cwd() + file.path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                // call function for move file 
                await utilitiesController.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, fileName);

                file.fileName = fileName;
                file.path = filepath;
            }))
            LeadResult.Leads.save();
        }

        return res.status(200).send({ status: true, LeadResult: LeadResult.Leads });
    }
    catch (e) {
        logger.error("Lead Controller 277. createLead: "+ e.toString());
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for create new lead 
 * @param {*} req 
 * @param {*} res 
 * @returns  Object LeadResult { status: true, Leads: Object }
 * -  { status: false, error: string }
 */
module.exports.createInquiryLead = async (req, res) => {
    try {
        const datas = req.local.lead;
        const orgId = req.local.lead.organizationId || req.local.user.organization;
        const leadType = (req.local.lead.leadType && req.local.lead.leadType.value) ? req.local.lead.leadType.value : "INQUIRY";
        datas.type = leadType;

        const Organizations = await OrganizationServices.getById(orgId);
        if (!Organizations.status || Organizations.error) return res.status(500).send(Organizations);
        datas.organization = Organizations.Organizations;

        if (datas.documents) {
            datas.documents.map((document) => {
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                document.id = doc_id;
                document.uploadedBy = req.local.user._id;
                document.createdAt = new Date();
            })
        }

        const case_result = await caseServices.find({ limit: 1 });
        if (!case_result.status || case_result.error) return res.status(500).send(case_result);

        if (case_result.cases.length == 0) {
            const str = "" + 1, pad = "000000"
            const answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }
        else {
            let caseNo = case_result.cases[0].caseNumber;
            let caseNumber = parseInt(caseNo) + 1;
            const str = "" + caseNumber, pad = "000000"
            const answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }

        datas.createdBy = req.local.user._id;
        let caseHistory = [];
        caseHistory = [{
            "id": Math.floor(100000 + Math.random() * 900000),
            "title": "Create Lead",
            "description": "Create Lead",
            "datetime": new Date(),
        }];
        if (datas.remark) {

            datas.leadRemarks = [{
                "id": Math.floor(100000 + Math.random() * 900000),
                "note": datas.remark,
                "createdAt": new Date(),
                createdBy: req.local.user
            }];
            delete datas.remark;
        }

        datas.caseHistory = caseHistory;
        datas.caseStatus = 'LEAD';

        const LeadResult = await LeadServices.createLead(datas);
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        // From here call createCaseLog function in casecontroller for create caseLog
        req.body.caseStatus = "CREATED " + leadType;
        const createCaseLog = await caseController.createCaseLog(req);

        const id = LeadResult.Leads._id.toString();
        if (datas.audioRemarks && datas.audioRemarks.length > 0) {
            const targetDirectory = path.join('cases', id, 'audioRemarks');
            let audioFiles = [];
            await Promise.all(datas.audioRemarks.map(async (base64) => {

                const fileName = 'audio_' + moment(new Date()).format("dd_mm_yyyy_hh_mm_ss") + ".m4a";
                const { status, filepath, error } = await base64toFile(base64, fileName, targetDirectory)
                if (status) {
                    audioFiles.push({
                        id: Math.floor(100000 + Math.random() * 900000),
                        fileName: fileName,
                        path: filepath,
                        uploadedBy: req.local.user._id,
                        createdAt: new Date(),
                    })
                }
            }));
            LeadResult.Leads.audioRemarks = audioFiles;
        }

        // ========================== Document, Image ==========================
        if (LeadResult.Leads.documents.length > 0) {

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
            }

            await Promise.all(LeadResult.Leads.documents.map(async file => {

                const fileName = "document_" + file.fileName;
                let currentPath = process.cwd() + file.path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                // call function for move file 
                await utilitiesController.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, fileName);

                file.fileName = fileName;
                file.path = filepath;
            }))
            LeadResult.Leads.save();
        }

        return res.status(200).send({ status: true, LeadResult: LeadResult.Leads });
    }
    catch (e) {
        logger.error("Lead Controller 402. createInquiryLead: "+ e.toString());
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for update Lead  
 * @param {*} req.params.id from that get leadId and from req.local.lead get lead object  
 * @param {*} res 
 * @returns object result { status: true, Leads: Object }
 * - { status: false, error: String }
 */
module.exports.updateLead = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const datas = req.local.lead;
        const user = req.local.user;

        if (datas.creditOfficer) {
            const creditOfficer = await CreditOfficersServices.findById(datas.creditOfficer);
            if (!creditOfficer.status || creditOfficer.error) return res.status(500).send(creditOfficer);
            datas.creditOfficer = creditOfficer.data;
        }
        // if (datas.organization) {
        //     const Organizations = await OrganizationServices.getById(orgId);
        //     if (!Organizations.status || Organizations.error) return res.status(500).send(Organizations);
        //     datas.organization = Organizations.Organizations;
        // }
        if (datas.client) {
            const Client = await ClientServices.getClientById(datas.client);
            if (!Client.status || Client.error) return res.status(500).send(Client);
            datas.client = Client.clients;
        }
        if (datas.branch) {
            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status || Branch.error) return res.status(500).send(Branch);
            datas.branch = Branch.branches;
        }
        if (datas.zone) {
            const Zone = await ZoneServices.getZone(datas.zone);
            if (!Zone.status || Zone.error) return res.status(500).send(Zone);
            datas.zone = Zone.zones;
        }
        if (datas.owners) {
            datas.owners.map((owner) => {
                const owner_id = Math.floor(100000 + Math.random() * 900000);
                owner.id = owner_id;
            })
        }

        if (datas.documents) {
            datas.documents.map((document) => {
                if (!document.id) {
                    const doc_id = Math.floor(100000 + Math.random() * 900000);
                    document.id = doc_id;
                    document.uploadedBy = req.local.user._id;
                    document.createdAt = new Date()
                }
                // check if file is in the case folder or not
                if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id + '/' + document.fileName)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });

                    const fileName = "document_" + document.fileName;
                    let currentPath = process.cwd() + document.path;

                    const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                    // call function for move file 
                    utilitiesController.moveFile(currentPath, destination);
                    const filepath = path.join("cases", id, fileName);

                    document.fileName = fileName;
                    document.path = filepath;
                }
            })
        }

        datas.updatedBy = req.local.user._id;

        const LeadResult = await LeadServices.updateLead(id, datas);
        if (!LeadResult.status) return res.status(500).send(LeadResult);

        // Create Task for advance invoice if true and is not created
        if (!!datas.isAdvanceInvoice && datas.isAdvanceInvoice) {
            const advanceInvoiceTaskRes = await TaskController.createTask({ organizationId: LeadResult.Leads.organization._id, caseId: LeadResult.Leads._id, workType: workType.PREPARE_INVOICE });
            //  call a request for raise make  invoice
            await nInvoiceController.createInvoiceRequestSystem({ caseId: LeadResult.Leads._id, user })
        }

        // From here call createCaseLog function in casecontroller for create caseLog
        req.body.caseStatus = "UPDATELEAD";
        const createCaseLog = await caseController.createCaseLog(req);

        return res.status(200).send(LeadResult);
    }
    catch (err) {
        logger.error("Lead Controller 500. updateLead: "+ err.toString());
        return res.status(500).send({ status: false, error: err.toString() });
    }
};

/**
 * @description This is used for update Lead  
 * @param {*} req.params.id from that get leadId and from req.local.lead get lead object  
 * @param {*} res 
 * @returns object result { status: true, Leads: Object }
 * - { status: false, error: String }
 */
module.exports.updateInquiryLead = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const datas = req.local.lead;
        if (datas.documents) {
            datas.documents.map((document) => {
                if (!document.id) {
                    const doc_id = Math.floor(100000 + Math.random() * 900000);
                    document.id = doc_id;
                    document.uploadedBy = req.local.user._id;
                    document.createdAt = new Date()
                }
                // check if file is in the case folder or not
                if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id + '/' + document.fileName)) {
                    fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });

                    const fileName = "document_" + document.fileName;
                    let currentPath = process.cwd() + document.path;

                    const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                    // call function for move file 
                    utilitiesController.moveFile(currentPath, destination);
                    const filepath = path.join("cases", id, fileName);

                    document.fileName = fileName;
                    document.path = filepath;
                }
            })
        }

        datas.updatedBy = req.local.user._id;

        const LeadResult = await LeadServices.updateLead(id, datas);
        const result = await LeadServices.getLead(id);
        if (!LeadResult) return res.status(500).send(LeadResult);
        if (!LeadResult.status) return res.status(500).send(LeadResult);

        // From here call createCaseLog function in casecontroller for create caseLog
        req.body.caseStatus = "UPDATELEAD";
        const createCaseLog = await caseController.createCaseLog(req);

        return res.status(200).send(result);
    }
    catch (err) {
        logger.error("Lead Controller 559. updateInquiryLead: "+ err.toString());
        return res.status(500).send({ status: false, error: err.toString() });
    }
};

/**
 * @description This is used for update Lead  
 * @param {*} req.params.id from that get leadId and from req.local.lead get lead object  
 * @param {*} res 
 * @returns object result { status: true, Leads: Object }
 * - { status: false, error: String }
 */
module.exports.addRemarkInquiryLead = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        let datas = req.local.lead;
        datas.updatedBy = req.local.user._id;
        console.log("🚀 ~ file: Lead.controller.js:581 ~ module.exports.addRemarkInquiryLead= ~ datas:", datas)

        if (datas.audioRemark && datas.audioRemark.trim() != "") {
            const targetDirectory = path.join('cases', id, 'audioRemarks');

            const fileName = 'audio_' + moment(new Date()).format("dd_mm_yyyy_hh_mm_ss") + ".m4a";
            const { status, filepath, error } = await base64toFile(datas.audioRemark, fileName, targetDirectory)
            if (status) {
                const audioFile = {
                    id: Math.floor(100000 + Math.random() * 900000),
                    fileName: fileName,
                    path: filepath,
                    createdBy: req.local.user._id,
                    createdAt: new Date(),
                }

                datas = {
                    ...datas,
                    $push: {
                        audioRemarks: audioFile
                    }
                }
            }
            delete datas.audioRemark;
        }
        if (datas.remark) {
            datas = {
                ...datas,
                $push: {
                    leadRemarks: {
                        "id": Math.floor(100000 + Math.random() * 900000),
                        "note": datas.remark,
                        "createdAt": new Date(),
                        createdBy: req.local.user
                    }
                }
            }
            delete datas.remark;
        }

        const LeadResult = await LeadServices.updateLead(id, datas);
        if (!LeadResult.status) return res.status(500).send(LeadResult);

        // From here call createCaseLog function in casecontroller for create caseLog
        req.body.caseStatus = "UPDATELEAD";
        const createCaseLog = await caseController.createCaseLog(req);

        return res.status(200).send(LeadResult);
    }
    catch (e) {
        logger.error("Lead Controller 628. addRemarkInquiryLead: "+ e.toString());
        return res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This is used for convert lead into Case   
 * @param {*} req.params.id from that get leadId 
 * @param {*} res 
 * @returns Object LeadResult  { status: true, Leads: Object }
 - { status: false, error: string }
 - { status: false, error: "Document is not Verified" }
 */
module.exports.convertCase = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });
        const user = req.local.user.toJSON();
        let datas = {};

        const LeadResult = await LeadServices.getLead(id);
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        if (LeadResult.Leads && !!LeadResult.Leads.createdBy) {
            const pickupPerson = await UserServices.findById(LeadResult.Leads.createdBy);
            if (!pickupPerson.status || pickupPerson.error) return res.status(500).send(pickupPerson);
            datas.staff = { pickupPerson: pickupPerson.users };
        }

        const LeadUpdateResult = await LeadServices.updateLead(id, { ...datas, type: "CASE", caseStatus: "NEW" });
        if (!LeadResult.status || LeadResult.error) return res.status(500).send(LeadResult);

        req.body.caseStatus = "CONVERTCASE";
        const createCaseLog = await caseController.createCaseLog(req);

        const geo = LeadResult.Leads.geo;

        // CREATE_TASK of case for DOC_VERIFY and SITE_INSPECTION
        const caseTaskRes = await TaskController.createTask({ caseId: id, organizationId: LeadResult.Leads.organization._id, workType: workType.CASE, users: [{ ...user, userId: user._id }], status: "COMPLETED", completedAt: new Date() });
        const rateVerifyTask = await TaskController.createTask({ organizationId: LeadResult.Leads.organization._id, caseId: LeadResult.Leads._id, workType: workType.RATE_VERIFY });
        const siteInspectionTask = await TaskController.createTask({ organizationId: LeadResult.Leads.organization._id, caseId: LeadResult.Leads._id, geo, workType: workType.SITE_INSPECTION });
        return res.status(200).send(LeadUpdateResult);
    }
    catch (err) {
        logger.error("🚀 ~ file: Lead.controller.js:672 ~ module.exports.convertCase= ~ err:"+ err.toString());
        return res.status(500).send({ status: false, error: err.toString() });
    }
};