const moment = require('moment');
const { dateRange } = require('../Utils/Utility');
const { logger } = require('../logger');

const CONFIG = require('../Constants/Config');
const LeaveServices = require('../Services/CreateServices')("Leave");
const AttendanceSummaryServices = require('../Services/CreateServices')("AttendanceSummary");

module.exports.report = async (req, res) => {
    try {
        const { user, startIndex, limit } = req.local;
        const { status, start, end, userId } = req.query;
        let query = {};

        if (["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1) {
            query = { userId: userId ? userId : user._id }
        } else {
            query = { userId: user._id }
        }

        if (status) { query.status = status };
        if (!!start && !!end) { query.date = dateRange(start, end) }

        const leaveRes = await LeaveServices.aggregate([
            {
                $match: query
            },
            {
                $group: {
                    _id: {
                        userId: "$userId",
                    },
                    leaves: {
                        $addToSet: '$$ROOT'
                    },
                    count: { $sum: 1 }
                }
            },
            {
                $lookup: {
                    from: "users",
                    localField: "_id.userId",
                    foreignField: "_id",
                    as: "userDetails"
                }
            },
            { $unwind: "$userDetails" },
            { $skip: startIndex },
            { $limit: limit }
        ]);
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);

        return res.status(200).send({ status: true, query, data: leaveRes.data });
    } catch (err) {
        logger.error("🚀 ~ file: Attendance.controller.js:11 ~ module.exports.getAll= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: e.toString() });
    }
}

module.exports.getAll = async (req, res) => {
    try {
        const { user, startIndex, limit } = req.local;
        const { leaveStatus, start, end, userId } = req.query;
        let query = {};

        if (["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1) {
            if (userId) {
                query = { userId }
            }
        } else {
            query = { userId: user._id }
        }

        if (leaveStatus) { query.leaveStatus = leaveStatus };
        if (!!start && moment(start, "DD-MM-YYYY").isValid()) {
            query.from = { $gte: new Date(moment(start, "DD-MM-YYYY").startOf('day')) }
        }
        if (!!end && moment(end, "DD-MM-YYYY").isValid()) {
            query.to = { $lte: new Date(moment(end, "DD-MM-YYYY").endOf('day')) }
        }

        const leaveRes = await LeaveServices.find({ query: query, limit: limit, start: startIndex, populate: ["userId"] });
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);

        return res.status(200).send({ status: true, query, data: leaveRes.data, count: leaveRes.count });
    } catch (err) {
        logger.error("🚀 ~ file: Attendance.controller.js:11 ~ module.exports.getAll= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: e.toString() });
    }
}

module.exports.get = async (req, res) => {
    try {
        const id = req.params.id;

        const leaveRes = await LeaveServices.find({ query: { _id: id }, populate: ["userId"] });
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);
        if (leaveRes.data.length == 0) return res.status(404).send({ status: false, error: "No Leave Found!" })

        return res.status(200).send({ status: true, data: leaveRes.data[0] });
    } catch (err) {
        logger.error("🚀 ~ file: Leave.controller.js ~ module.exports.get= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: "Internal Server error." });
    }
}

module.exports.createLeave = async (req, res) => {
    try {
        const data = req.local.leave;
        let { userId } = req.query;

        if (!userId) {
            userId = req.local.user._id.toString();
        }

        const leaveRes = await LeaveServices.create({ ...data, userId });
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);

        return res.status(200).send({ status: true, data: leaveRes.data });
    } catch (err) {
        logger.error("🚀 ~ file: Leave.controller.js ~ module.exports.createLeave= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: "Internal Server error." });
    }
}

module.exports.updateLeave = async (req, res) => {
    try {
        const id = req.params.id;
        const user = req.local.user;
        const data = req.local.leave;
        const hasPermission = ["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1;

        const leaveRes = await LeaveServices.findById(id);
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);

        if (leaveRes.data.leaveStatus === "PENDING") {
            if (leaveRes.data.leaveStatus) {
                data.approvedBy = user;
                data.approvedAt = new Date();
            }

            const updateLeaveRes = await LeaveServices.updateById(id, { ...data });
            if (!updateLeaveRes.status || updateLeaveRes.error) return res.status(500).send(updateLeaveRes);

            return res.status(200).send({ status: true, data: updateLeaveRes.data });
        }
        else {
            return res.status(500).send({ status: false, error: "can't update leave after approved/rejected!" });
        }

    } catch (err) {
        logger.error("file: Leave.controller.js ~ module.exports.updateLeaveStatus= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: "Internal Server error." });
    }
}

module.exports.deleteLeave = async (req, res) => {
    try {
        const id = req.params.id;
        const user =  req.local.user;
        const hasPermission = ["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) != -1;

        const leaveRes = await LeaveServices.findById(id);
        if (!leaveRes.status || leaveRes.error) return res.status(500).send(leaveRes);

        if (leaveRes.data.leaveStatus === "PENDING" || hasPermission) {
            const leaveRes = await LeaveServices.deleteById(id);
            return res.status(leaveRes.status ? 200 : 500).send(leaveRes);
        }
        else {
            return res.status(500).send({ status: false, error: "can't delete leave after approved/rejected" });
        }
    } catch (err) {
        logger.error("file: Leave.controller.js ~ module.exports.updateLeaveStatus= ~ e:" + JSON.stringify(err));
        return res.status(200).send({ status: false, error: "Internal Server error." });
    }
}