
const joi = require('@hapi/joi');
const { logger } = require('../logger');

/**
 * @description This is used for validate request for create Organization
 * @param {Object} req  
 * @param {*} res 
 * @param {*} next i.e. next to the controller
 */
exports.createOrganizationValidation = async (req, res, next) => {
    let data = req.body;

    if (data.contact && typeof data.contact == "string") data.contact = JSON.parse(data.contact);
    if (data.owners && typeof data.owners == "string") data.owners = JSON.parse(data.owners);
    if (data.branch && typeof data.branch == "string") data.branch = JSON.parse(data.branch);
    if (data.bankDetails && typeof data.bankDetails == "string") data.bankDetails = JSON.parse(data.bankDetails);

    const schema = joi.object({
        name: joi.string().min(3).required(),
        shortName: joi.string(),
        email: joi.string().email().required(),
        panNo: joi.string(),
        gstNo: joi.string(),
        mobileNo: joi.string(),

        contact: joi.object({
            name: joi.string(),
            email: joi.string(),
            mobileNo: joi.string()
        }),

        // branch: joi.object({
        //     name: joi.string().min(3).required(),
        //     shortName: joi.string(),
        //     email: joi.string().email().required(),
        //     panNo: joi.string(),
        //     gstNo: joi.string(),
        //     mobileNo: joi.string(),
        //     city: joi.string().allow(''),
        //     state: joi.string().allow(''),
        //     address: joi.string(),
        // }),

        city: joi.string().allow(''),
        state: joi.string().allow(''),
        address: joi.string(),

        package: joi.string().allow(''),

        owners: joi.array(),

        documents: joi.array(),
        bankDetails: joi.object({
            bankName: joi.string().allow(''),
            accountNo: joi.string().allow(''),
            ifscCode: joi.string().allow(''),
            beneficiaryName: joi.string().allow('')
        }),

        organization: joi.string().allow(''),

    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, organization: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("🚀 ~ file: validation.js:50 ~ exports.createOrganizationValidation= ~ err:", errors)
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
};


/**
 * @description This is used for validate request for create Organization
 * @param {Object} req  
 * @param {*} res 
 * @param {*} next i.e. next to the controller
 */
exports.createOrgBranchValidation = async (req, res, next) => {
    let data = req.body;

    if (data.owners && typeof data.owners == "string") data.owners = JSON.parse(data.owners);
    if (data.bankDetails && typeof data.bankDetails == "string") data.bankDetails = JSON.parse(data.bankDetails);

    const schema = joi.object({
        name: joi.string().min(3).required(),
        shortName: joi.string(),
        email: joi.string().email().required(),
        panNo: joi.string(),
        gstNo: joi.string(),
        mobileNo: joi.string(),
        city: joi.string().allow(''),
        state: joi.string().allow(''),
        address: joi.string(),

        logo: joi.string().allow(''),
        package: joi.string().allow(''),

        owners: joi.array(),

        documents: joi.array(),
        bankDetails: joi.object({
            bankName: joi.string(),
            accountNo: joi.string(),
            ifscCode: joi.string(),
            beneficiaryName: joi.string()
        }),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, branch: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("🚀 ~ file: validation.js:119 ~ exports.createOrgBranchValidation= ~ errors:", errors)
            res.status(409).send({ status: false, error: errors, body: req.body });
        });

};

exports.createTask = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        userId: joi.string(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, task: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("🚀 ~ file: validation.js:141 ~ exports.createTask= ~ errors:", errors)
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.clientVisitTask = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        userId: joi.string(),
        location: joi.object(),
        workType: joi.string().required(),
        action: joi.string(),
        clientType: joi.object(),
        clientId: joi.string().allow(''),
        branchId: joi.string().allow(''),
        clientName: joi.string().allow(''),
        creditOfficerId: joi.object(),
        remark: joi.string(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, task: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("🚀 ~ file: validation.js:141 ~ exports.createTask= ~ errors:", errors)
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.clientVisitUpdateTask = async (req, res, next) => {
    let data = req.body;
    if (data.audioNotes && typeof data.audioNotes == "string") data.audioNotes = JSON.parse(data.audioNotes);

    const schema = joi.object({
        action: joi.string(),
        location: joi.object(),
        creditOfficerId: joi.string(),
        remark: joi.string().allow("", null),
        audioNotes: joi.array()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            console.log("data: ", data);
            req.local = { ...req.local, task: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("🚀 ~ file: validation.js:141 ~ exports.createTask= ~ errors:", errors)
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

/**
 * @description This is used for validate Login Object 
 * @param {Object} req 
 * @param {*} res 
 * @param {*} next 
 */
exports.loginValidation = async (req, res, next) => {

    const schema = joi.object({
        email: joi.string().email().required(),
        password: joi.string().min(6).required(),
        platform: joi.string()
    });

    schema.validateAsync(req.body, { abortEarly: false })
        .then(result => {
            next();
        }).catch(err => {
            console.log(err)
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
};

exports.createUserValidation = async (req, res, next) => {
    const user = req.body;
    // console.log(user);

    if (user.bankDetails && typeof user.bankDetails == "string") user.bankDetails = JSON.parse(user.bankDetails);
    if (user.address && typeof user.address == "string") user.address = JSON.parse(user.address);
    if (user.goodPropertyType && typeof user.goodPropertyType == "string") user.goodPropertyType = JSON.parse(user.goodPropertyType);
    if (user.permissions && typeof user.permissions == "string") user.permissions = JSON.parse(user.permissions);
    if (user.registrationNo && typeof user.registrationNo == "string") user.registrationNo = JSON.parse(user.registrationNo);
    if (user.employeeDocs && typeof user.employeeDocs == "string") user.employeeDocs = JSON.parse(user.employeeDocs);
    if (user.removeImg && typeof user.removeImg == "string") user.removeImg = JSON.parse(user.removeImg);
    if (user.orgBranches && typeof user.orgBranches == "string") user.orgBranches = JSON.parse(user.orgBranches);

    // console.table(user.goodPropertyType);
    // return res.send({ test: true, user })
    const schema = joi.object({
        firstName: joi.string().min(2).required(),
        lastName: joi.string(),
        registrationNo: joi.array().allow(''),
        type: joi.string().allow(''),
        employeeDocs: joi.array(),
        removeImg: joi.array(),
        employeeCode: joi.string().allow(''),
        email: joi.string().email().required(),
        mobileNo: joi.string().allow(''),
        workMobileNo: joi.string().allow(''),
        emergencyMobileNo: joi.string().allow(''),
        fax: joi.string().allow(''),
        designation: joi.string().allow(''),
        qualification: joi.string().allow(''),
        goodPropertyType: joi.array().allow(''),
        workWithDocumentMaker: joi.string().allow(''),
        shortName: joi.string().allow(''),
        fatherName: joi.string(),
        PANNo: joi.string().allow(''),
        birthDate: joi.string().allow(''),
        incentiveAccount: joi.string(),
        bankDetails: joi.object({
            bankName: joi.string().allow(''),
            branchName: joi.string().allow(''),
            accountNo: joi.string().allow(''),
            ifscCode: joi.string().allow(''),
            benificaryName: joi.string().allow('')
        }),
        permissions: joi.object(),
        taskLimit: joi.string(),
        address: joi.object({
            street: joi.string().allow(''),
            city: joi.string().allow(''),
            state: joi.string().allow(''),
            country: joi.string().allow(''),
            postalCode: joi.string().allow(''),
            description: joi.string().allow('')
        }),
        organization: joi.string(),
        orgBranches: joi.object().allow(),

        branch: joi.string(),
        password: joi.string(),
        role: joi.string(),
    });

    schema.validateAsync(user, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, users: user };

            next();
        }).catch(err => {

            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.updateUserProfileValidation = async (req, res, next) => {
    const user = req.body;

    // console.log(user, "===== validation");

    if (user.bankDetails && typeof user.bankDetails == "string") user.bankDetails = JSON.parse(user.bankDetails);
    if (user.address && typeof user.address == "string") user.address = JSON.parse(user.address);
    if (user.goodPropertyType && typeof user.goodPropertyType == "string") user.goodPropertyType = JSON.parse(user.goodPropertyType);
    if (user.bankDetails && typeof user.bankDetails == "string") user.bankDetails = JSON.parse(user.bankDetails);
    // if (user.registrationNo && typeof user.registrationNo == "string") user.registrationNo = JSON.parse(user.registrationNo);
    if (user.employeeDocs && typeof user.employeeDocs == "string") user.employeeDocs = JSON.parse(user.employeeDocs);
    if (user.removeImg && typeof user.removeImg == "string") user.removeImg = JSON.parse(user.removeImg);

    // console.log("user", user.goodPropertyType)
    const schema = joi.object({
        firstName: joi.string().min(2).required(),
        lastName: joi.string().min(3).required(),
        // email: joi.string().email().required(),
        mobileNo: joi.string().allow(''),
        type: joi.string().allow(''),
        emergencyMobileNo: joi.string().allow(''),
        workMobileNo: joi.string().allow(''),
        employeeDocs: joi.array(),
        removeImg: joi.array(),
        fax: joi.string().allow(''),
        qualification: joi.string().allow(''),
        goodPropertyType: joi.array().allow(''),
        workWithDocumentMaker: joi.string().allow(''),
        // shortName: joi.string().allow(''),
        // registrationNo: joi.array().allow(""),
        fatherName: joi.string(),
        PANNo: joi.string(),
        birthDate: joi.string().allow(''),
        incentiveAccount: joi.string(),
        bankDetails: joi.object({
            bankName: joi.string().allow(''),
            branchName: joi.string().allow(''),
            accountNo: joi.string().allow(''),
            ifscCode: joi.string().allow(''),
            benificaryName: joi.string().allow('')
        }),
        designation: joi.string().allow(''),
        address: joi.object({
            // street: joi.string().allow(''),
            city: joi.string().allow(''),
            state: joi.string().allow(''),
            country: joi.string().allow(''),
            postalCode: joi.string().allow(''),
            description: joi.string().allow('')
        }),
        // profilePhoto: joi.string(),
        branch: joi.string(),
        password: joi.string(),
        role: joi.string(),
        taskLimit: joi.string(),
    });

    schema.validateAsync(user, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, users: user };
            // console.log(user, "  validation");
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}
exports.organizationStatus = async (req, res, next) => {
    const schema = joi.object({
        status: joi.boolean().required()
    });

    schema.validateAsync(req.body, { abortEarly: false })
        .then(result => {
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.branchValidation = async (req, res, next) => {
    const data = req.body;
    // if (data.valuers) data.valuers = JSON.parse(data.valuers);
    const schema = joi.object({


        name: joi.string().min(3),
        branchCode: joi.string().allow(""),
        zone: joi.string().allow(''),
        type: joi.string(),
        client: joi.string(),
        typeOfFees: joi.string(),// CUSTOM / STANDARD
        fees: joi.object(),
        assignmentType: joi.object(),
        mobileNo: joi.string().required(),
        alternateMobileNo: joi.string().allow(""),
        email: joi.string().email().required(),
        sendStructuralCertificate: joi.string(),
        sendStructuralReport: joi.string(),
        description: joi.string(),
        address: joi.string(),
        fax: joi.string().allow(""),
        ledgerName: joi.string().allow(""),
        valuers: joi.array(),
        pickupPerson: joi.string().allow(""),
        creditOfficers: joi.array()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, branch: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.clientValidation = async (req, res, next) => {
    let data = req.body;
    if (data.address && typeof data.address == "string") data.address = JSON.parse(data.address);

    const schema = joi.object({
        logo: joi.string().allow('', null),
        name: joi.string().min(3),
        phone: joi.string().allow(''),
        displayName: joi.string().allow(''),
        initial: joi.string().allow(''),
        type: joi.string(),
        remark: joi.string().allow(''),
        uniqueId: joi.string().allow(''),
        ledgerName: joi.string().allow(''),
        // valuers: joi.array(),
        email: joi.string().email().allow(''),
        address: joi.object().allow(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, client: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.inquiryLeadValidation = async (req, res, next) => {
    const data = req.body;
    console.log("🚀 ~ file: validation.js:389 ~ exports.inquiryLeadValidation= ~ data:", data)
    if (data.leadType && typeof data.leadType == "string") data.leadType = JSON.parse(data.leadType);
    if (data.audioRemarks && typeof data.audioRemarks == "string") data.audioRemarks = JSON.parse(data.audioRemarks);

    let schemaObj = {
        leadType: joi.object(),
        leadFrom: joi.object(),
        inquiryTo: joi.string().required(),
        ownerName: joi.string().required(),
        ownerPhoneNo: joi.string().allow(''),
        email: joi.string().allow(''),
        appAddress: joi.string().allow(''),
        remark: joi.string().allow(''),
        documents: joi.array(),
        audioRemarks: joi.array(),
    };

    const schema = joi.object(schemaObj);

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, lead: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.addRemarkInquiryLead = async (req, res, next) => {
    const data = req.body;
    if (data.leadType && typeof data.leadType == "string") data.leadType = JSON.parse(data.leadType);
    if (data.audioRemarks && typeof data.audioRemarks == "string") data.audioRemarks = JSON.parse(data.audioRemarks);

    let schemaObj = {
        remark: joi.string().allow(''),
        audioRemark: joi.string().allow(''),
    };

    const schema = joi.object(schemaObj);

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, lead: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.leadValidation = async (req, res, next) => {
    const data = req.body;
    if (data.leadType && typeof data.leadType == "string") data.leadType = JSON.parse(data.leadType);
    if (data.audioRemarks && typeof data.audioRemarks == "string") data.audioRemarks = JSON.parse(data.audioRemarks);

    let schemaObj = {
        leadType: joi.object(),
        organizationId: joi.string(),
        creditOfficer: joi.string().allow(''),
        branch: joi.string().allow(''),
        client: joi.string().allow(''),
        zone: joi.string().allow(''),
        fullAddress: joi.string().allow(''),
        appAddress: joi.string().allow(''),
        addressForMap: joi.string(),
        caseNumber: joi.string(),
        typeOfReport: joi.object().required(),
        siteVisitContactNo: joi.string(),
        contactPersonName: joi.string(),
        relationWithOwner: joi.string(),
        location: joi.object({
            latitude: joi.string(),
            longitude: joi.string()
        }),
        purchaseValueAsPerDocument: joi.string().allow(''),
        leadFrom: joi.object(),
        purposeOfValuation: joi.object().allow(''),
        toAssess: joi.object(),
        masterTypeOfPurpose: joi.object(),
        sourceOfClient: joi.string(),
        caseReferredBy: joi.string().allow(''),
        propertyType: joi.object({
            label: joi.string(),
            value: joi.string()
        }),
        assetsClass: joi.object(),
        nameOfClient: joi.string(),
        remark: joi.string().allow(''),
        clientType: joi.object(),
        documents: joi.array(),
        address: joi.object(),
        ownerName: joi.string(),
        ownerPhoneNo: joi.string(),
        ownershipShare: joi.string().allow(''),
        email: joi.string().allow(''),
        owners: joi.array(),
        proposePurchasers: joi.array(),
        proposePurchaserName: joi.string().allow(''),
        staff: joi.object({
            pickupPerson: joi.string(),
            deliveryPerson: joi.string(),
            maker: joi.string(),
            checker: joi.string(),
            rateVerifier: joi.string(),
            siteEngineer: joi.string(),
            valuers: joi.array(),
        }),
        valuersRegistrationNumber: joi.object(),
        composition: joi.array(),
        otherComposition: joi.array(),
        outgoingDetail: joi.object(),
        isAdvanceInvoice: joi.boolean(),
        gstNo: joi.when('isAdvanceInvoice', { is: true, then: joi.string().allow(''), otherwise: joi.optional() }),
        invoiceAmount: joi.when('isAdvanceInvoice', { is: true, then: joi.required(), otherwise: joi.optional() }),
        invoiceType: joi.when('isAdvanceInvoice', { is: true, then: joi.string().allow(''), otherwise: joi.optional() }),
        // flatSpecification: joi.object(),
        fees: joi.when('leadType.value', { is: "LEAD", then: joi.object(), otherwise: joi.optional() }),
        assignmentType: joi.object(),
        uniqueNo: joi.string().allow('')
    };

    const schema = joi.object(schemaObj);

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, lead: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });

}

exports.caseValidation = async (req, res, next) => {
    const data = req.body;
    // console.log(" update Case :  ", JSON.stringify(data));
    const schema = joi.object({
        organizationId: joi.string(),
        typeOfReport: joi.object(),
        clientType: joi.object(),
        client: joi.string().allow(''),
        zone: joi.string().allow(''),
        branch: joi.string().allow(''),
        fullAddress: joi.string(),
        appAddress: joi.string().allow(''),
        addressForMap: joi.string().allow(''),
        caseNumber: joi.string(),
        creditOfficer: joi.string().allow(''),
        siteVisitContactNo: joi.string(),
        contactPersonName: joi.string(),
        location: joi.object({
            latitude: joi.number().label('Latitude'),
            longitude: joi.number().label('Longitude')
        }),
        purchaseValueAsPerDocument: joi.string().allow(''),
        leadFrom: joi.object(),
        purposeOfValuation: joi.object().allow(''),
        masterTypeOfPurpose: joi.object(),
        toAssess: joi.object(),
        sourceOfClient: joi.string(),
        caseReferredBy: joi.string().allow(''),
        propertyType: joi.object({
            label: joi.string(),
            value: joi.string()
        }),
        assetsClass: joi.object(),
        images: joi.array(),
        relationWithOwner: joi.string(),
        nameOfClient: joi.string(),
        remark: joi.string().allow(''),
        fees: joi.object(),
        assignmentType: joi.object(),
        documents: joi.array(),
        siteVisit: joi.object(),
        isUrgent: joi.boolean(),
        isExternal: joi.boolean(),
        isExternalReport: joi.boolean(),
        documentPerArea: joi.array(),
        address: joi.object(),
        ownerName: joi.string(),
        ownerPhoneNo: joi.string(),
        ownershipShare: joi.string().allow(''),
        email: joi.string().allow(''),
        owners: joi.array(),
        proposePurchaserName: joi.string().allow(''),
        proposePurchasers: joi.array(),
        reportInformation: joi.object(),
        isSiteMeasurementAllowed: joi.bool(),
        finalPlinthArea: joi.object(),
        staff: joi.object({
            pickupPerson: joi.string().allow('').allow(null),
            deliveryPerson: joi.string().allow('').allow(null),
            maker: joi.string().allow('').allow(null),
            rateVerifier: joi.string().allow('').allow(null),
            checker: joi.string().allow('').allow(null),
            siteEngineer: joi.string().allow('').allow(null),
            valuers: joi.array().allow(),
            valuer: joi.array().allow(),
        }),
        valuersRegistrationNumber: joi.object(),
        boundries: joi.object(),

        flatRate: joi.string(),
        buildingServicesRate: joi.string(),
        landOtherRate: joi.string(),
        propertyValuation: joi.string(),
        valuationAreaType: joi.string(),
        sourceValuation: joi.string(),

        composition: joi.array(),
        otherComposition: joi.array(),
        flatSpecifications: joi.object(),
        extraDetailOfProperty: joi.string().allow(''),
        briefDescOfProperty: joi.string(),
        general: joi.object(),
        compositeRate: joi.object(),
        extendOfSiteRemark: joi.string().allow(''),
        siteMeasurementRemark: joi.string().allow(''),
        indicatorArr: joi.array(),
        guidelineRate: joi.string(),
        siteMeasurement: joi.object(),
        outgoingDetail: joi.object(),
        marketability: joi.object(),
        constructionDetails: joi.object(),
        carpetTotal: joi.string(),
        otherCarpetTotal: joi.string(),
        toBeAdded: joi.string().allow(''),
        builtupPercentage: joi.string(),
        salablePercentage: joi.string(),
        finalCarpetArea: joi.string(),
        finalSalableArea: joi.string(),
        finalCarpetArea: joi.string(),
        routeMap: joi.object(),
        finalArea: joi.string(),
        finalValuationArea: joi.string(),
        finalAreaType: joi.string(),
        finalMeasurementType: joi.string().allow(''),
        finalAsPerArea: joi.string(),
        finalValuationAreaString: joi.string(),
        extraAreaTotalCount: joi.string().optional(),
        finalBuiltupArea: joi.string(),
        valuation: joi.object(),
        reportImages: joi.object(),
        externalImages: joi.array(),
        marketAnalysisAndRemark: joi.object(),

        landDetails: joi.object(),
        shopDetails: joi.object(),
        landAndBuildingDetails: joi.object(),
        reraId: joi.string().allow(''),
        apfDetails: joi.object(),
        factorFavoringValue: joi.object(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, case: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            logger.error(`🚀 ~ file: validation.js:648 ~ exports.caseValidation= ~ errors:  ${errors.toString()}`)
            res.status(500).send({ status: false, error: errors });
        });

}

exports.caseValuationValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        // typeOfReport: joi.object().required(),
        // clientType: joi.object(),
        // client: joi.string(),
        // zone: joi.string().allow(''),
        // branch: joi.string(),
        fullAddress: joi.string(),
        appAddress: joi.string().allow(''),
        addressForMap: joi.string().allow(''),
        // caseNumber: joi.string(),
        // creditOfficer: joi.string(),
        siteVisitContactNo: joi.string(),
        contactPersonName: joi.string(),
        relationWithOwner: joi.string(),
        location: joi.object({
            latitude: joi.string(),
            longitude: joi.string()
        }),
        // purchaseValueAsPerDocument: joi.string(),
        // leadFrom: joi.object(),
        propertyType: joi.object(),
        purposeOfValuation: joi.object(),
        masterTypeOfPurpose: joi.object(),
        toAssess: joi.object(),
        // sourceOfClient: joi.string(),
        // caseReferredBy: joi.string().allow(''),
        // propertyType: joi.object({
        //     label: joi.string(),
        //     value: joi.string()
        // }),
        // assetsClass: joi.object(),
        // images: joi.array(),
        // relationWithOwner: joi.string(),
        // nameOfClient: joi.string(),
        // remark: joi.string(),
        // // fees: joi.object({
        // //     type: joi.string().required(),
        // //     propertyValue: joi.string().required(),
        // //     discussWithPerson: joi.string().allow(""),
        // //     expectedFees: joi.number(),
        // //     reason: joi.string(),
        // //     quotedAmount: joi.string()
        // // }),
        isSiteMeasurementAllowed: joi.bool(),
        siteInspection: joi.array(),
        documents: joi.array(),
        documentPerArea: joi.array(),
        finalPlinthArea: joi.object(),
        // isUrgent: joi.boolean().allow(''),
        // isExternal: joi.boolean().allow(''),
        address: joi.object(),
        ownerName: joi.string(),
        ownerPhoneNo: joi.string(),
        ownershipShare: joi.string().allow(''),
        email: joi.string().allow(''),
        owners: joi.array(),
        proposePurchasers: joi.array(),
        proposePurchaserName: joi.string().allow(''),
        staff: joi.object({
            pickupPerson: joi.string().allow(''),
            deliveryPerson: joi.string().allow(''),
            maker: joi.string().allow(''),
            rateVerifier: joi.string().allow(''),
            checker: joi.string().allow(''),
            siteEngineer: joi.string().allow(''),
            valuers: joi.array().allow(),
            valuer: joi.array().allow(),
        }),
        boundries: joi.object(),
        flatRate: joi.string(),
        buildingServicesRate: joi.string(),
        landOtherRate: joi.string(),
        propertyValuation: joi.string(),
        valuationAreaType: joi.string(),
        sourceValuation: joi.string(),
        igrRate: joi.object(),
        igrFinalRate: joi.object(),
        otherRateDetails: joi.object(),
        compositionFlatType: joi.object(),
        composition: joi.array(),
        otherComposition: joi.array(),
        otherCompositionCarpetTotal: joi.string(),
        otherCompositionBuiltUpTotal: joi.string(),
        otherCompositionSaleableTotal: joi.string(),
        flatSpecifications: joi.object(),
        extraDetailOfProperty: joi.string().allow(''),
        briefDescOfProperty: joi.string(),
        general: joi.object(),
        compositeRate: joi.object(),
        extendOfSiteRemark: joi.string().allow(''),
        siteMeasurementRemark: joi.string().allow(''),
        indicatorArr: joi.array(),
        guidelineRate: joi.string(),
        siteMeasurement: joi.object(),
        outgoingDetail: joi.object(),
        marketability: joi.object(),
        constructionDetails: joi.object(),
        carpetTotal: joi.string(),
        otherCarpetTotal: joi.string(),
        toBeAdded: joi.string().allow(''),
        builtupPercentage: joi.string(),
        salablePercentage: joi.string(),
        finalCarpetArea: joi.string(),
        finalSalableArea: joi.string(),
        finalCarpetArea: joi.string(),
        routeMap: joi.object(),
        finalArea: joi.string(),
        finalValuationArea: joi.string(),
        finalAreaType: joi.string(),
        finalMeasurementType: joi.string().allow(''),
        finalAsPerArea: joi.string(),
        finalValuationAreaString: joi.string(),
        finalCarpetConvertToBuiltUpPer: joi.string().allow(''),
        finalBuiltUpPercentage: joi.string().allow(''),
        finalSaleablePercentage: joi.string().allow(''),
        extraAreaTotalCount: joi.string().optional(),
        finalBuiltupArea: joi.string(),
        valuation: joi.object(),
        reportImages: joi.object(),
        externalImages: joi.array(),
        rateVerify: joi.object().allow(''),
        underConstruction: joi.object().allow(''),
        guidelineDetails: joi.object(),
        marketAnalysisAndRemark: joi.object(),
        indicatorRange: joi.object(),
        landDetails: joi.object(),
        shopDetails: joi.object(),
        reportInformation: joi.object(),
        landAndBuildingDetails: joi.object(),
        reraId: joi.string().allow(''),
        apfDetails: joi.object(),
        factorFavoringValue: joi.object(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, caseValuation: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.caseUpdateDateValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        siteVisitedDate: joi.string().allow(''),
        reportGeneratedDate: joi.string().allow(''),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, dates: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.caseExternalDetailsValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        location: joi.object({
            latitude: joi.string(),
            longitude: joi.string()
        }),
        externalReportDetails: joi.object({
            area: joi.string(),
            rate: joi.string(),
            yearOfConstruction: joi.string(),
            compositionOfBuilding: joi.string(),
            finalMeasurementType: joi.string().allow(""),
            finalAreaType: joi.string().allow(""),
            underConstruction: joi.boolean(),
        }),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, externalDetails: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.casePartialInspectionValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        location: joi.object({
            latitude: joi.string(),
            longitude: joi.string()
        }),
        remark: joi.string().allow(''),
        taskId: joi.string().allow(''),
        documents: joi.array(),
        externalImages: joi.array(),
        address: joi.object()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, caseValuation: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.updateInvoiceValidation = async (req, res, next) => {
    const data = req.body;
    if (data.cases && typeof data.cases == "string") data.cases = JSON.parse(data.cases);
    if (data.voucher && typeof data.voucher == "string") data.voucher = JSON.parse(data.voucher);

    const schemaJSON = {
        basic: joi.string().allow(""),
        gst: joi.string().allow(""),
        ops: joi.string().allow(""),
        tds: joi.string().allow(""),
        invoiceNo: joi.string(),
        amount: joi.string(),
        invoiceType: joi.string().allow(""),
        voucher: joi.object().allow(null),
        cases: joi.array().min(1).message("Select at least One case"),
    };

    const schema = joi.object(schemaJSON);

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, invoice: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            console.log("=>i invoice upload: " + JSON.stringify(data, null, 2) + "\n\n" + JSON.stringify(errors, null, 2));
            logger.error("=>i invoice upload: " + JSON.stringify(data, null, 2) + "\n\n" + JSON.stringify(errors, null, 2))
            res.status(500).send({ status: false, error: errors });
        });
}

exports.addPayment = async (req, res, next) => {
    const data = req.body;
    if (data.caseIds && typeof data.caseIds == "string") data.caseIds = JSON.parse(data.caseIds);
    if (data.invoiceIds && typeof data.invoiceIds == "string") data.invoiceIds = JSON.parse(data.invoiceIds);
    if (data.completedCaseIds && typeof data.completedCaseIds == "string") data.completedCaseIds = JSON.parse(data.completedCaseIds) || [];

    const schema = joi.object({
        caseIds: joi.array().required(),
        paymentType: joi.string().required(),
        note: joi.string().allow(''),
        number: joi.string().allow(''),
        amount: joi.string(),
        discount: joi.string().allow(''),
        date: joi.string().allow('').allow(null),
        status: joi.string().allow(''),
        invoiceIds: joi.array().allow(),
        completedCaseIds: joi.array().optional(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, payment: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.dropDownValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({

        name: joi.string().required(),
        // organization: joi.string(),
        options: joi.array()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, dropDown: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.packageValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        title: joi.string(),
        price: joi.number(),
        case: joi.number(),
        duration: joi.string(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, package: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.LocationHistoryValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        userId: joi.string(),
        location: joi.object(),
        otherDetails: joi.object(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, LocationHistory: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.zoneValidation = async (req, res, next) => {
    const data = req.body;

    // console.log("In Validation : ", JSON.stringify(data));
    const schema = joi.object({

        client: joi.string(),
        zoneName: joi.string(),
        empanelments: joi.array(),

    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, zone: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.createContactValidation = async (req, res, next) => {
    const data = req.body;

    const schema = joi.object({

        type: joi.string(),
        details: joi.string()

    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, createContact: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.reportTemplateValidation = async (req, res, next) => {
    const data = req.body;

    const schema = joi.object({
        client: joi.string(),
        branch: joi.string().allow(''),
        zone: joi.string().allow(''),
        propertyType: joi.string(),
        typeOfReport: joi.string(),
        fileName: joi.string(),
        workStatus: joi.string(),
        htmlCode: joi.string().allow(""),
        header: joi.string().allow(""),
        footer: joi.string().allow(""),
        status: joi.boolean()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, reportTemplate: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.assetValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        name: joi.string(),
        assetNo: joi.string(),
        assetImage: joi.string().allow(''),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, asset: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.reminderValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        name: joi.string(),
        dateTime: joi.string(),
        description: joi.string(),
        createdBy: joi.string(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, reminder: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.holidayValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        title: joi.string(),
        dateTime: joi.string(),
        status: joi.boolean(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, holiday: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.dynamicFormValidation = async (req, res, next) => {
    const data = req.body;
    const schema = joi.object({
        clientId: joi.string(),
        description: joi.string(),
        constructionWorkStatus: joi.string(),
        module: joi.object(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, dynamicForm: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.invoiceCreateValidation = async (req, res, next) => {
    const data = req.body;
    const schemaJSON = {
        caseId: joi.string().allow(''),
        invoiceType: joi.string().valid('TAX', 'ADVANCE', 'PROFORMA'),
        gstNo: joi.string().allow(""),
        invoiceAmount: joi.string(),
        amount: joi.string()
    };

    const schema = joi.object(schemaJSON);

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, invoice: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.reviewTaskValidation = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        workType: joi.string(),
        user: joi.string().allow(''),
        reason: joi.string().allow('')
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, task: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.bugTicketValidation = async (req, res, next) => {
    let data = req.body;
    if (data.audioNotes && typeof data.audioNotes == "string") data.audioNotes = JSON.parse(data.audioNotes);

    const schema = joi.object({
        title: joi.string(),
        log: joi.string(),
        status: joi.string(),
        description: joi.string(),
        audioNotes: joi.array()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, bugTicket: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.ApprovalRequestValidation = async (req, res, next) => {
    let data = req.body;
    if (data.audioNotes && typeof data.audioNotes == "string") data.audioNotes = JSON.parse(data.audioNotes);

    const schema = joi.object({
        type: joi.string(),
        remark: joi.string(),
        caseId: joi.string(),
        taskId: joi.string(),
        workType: joi.string()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, data: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.masterData = async (req, res, next) => {
    let data = req.body;
    if (data.audioNotes && typeof data.audioNotes == "string") data.audioNotes = JSON.parse(data.audioNotes);

    const schema = joi.object({
        name: joi.string(),
        data: joi.any(),
        description: joi.string()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, masterData: data };

            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.addCaseRemark = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        note: joi.string(),
        taskId: joi.string(),
        workType: joi.string(),
        caseStatus: joi.string().optional(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, masterData: data };

            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.updatePayment = async (req, res, next) => {
    let data = req.body;
    if (data.tds && typeof data.tds == "string") data.tds = JSON.parse(data.tds);
    if (data.caseIds && typeof data.caseIds == "string") data.caseIds = JSON.parse(data.caseIds);
    if (data.completedCaseIds && typeof data.completedCaseIds == "string") data.completedCaseIds = JSON.parse(data.completedCaseIds);
    if (data.invoiceIds && typeof data.invoiceIds == "string") data.invoiceIds = JSON.parse(data.invoiceIds);
    if (data.voucher && typeof data.voucher == "string") data.voucher = JSON.parse(data.voucher);
    if (data.paymentSettlement && typeof data.paymentSettlement == "string") data.paymentSettlement = JSON.parse(data.paymentSettlement);

    const schema = joi.object({
        caseIds: joi.array(),
        completedCaseIds: joi.array(),
        paymentType: joi.string(),
        note: joi.string().allow(''),
        number: joi.string().allow(''),
        amount: joi.string(),
        discount: joi.string().allow(''),
        date: joi.string().allow('').allow(null),
        status: joi.string().allow(''),
        invoiceIds: joi.array().allow(),
        voucher: joi.object().allow(),
        paymentSettlement: joi.array(),
        type: joi.string(),
        remark: joi.string().allow(),
        tds: joi.object().allow(),
        tdsStatus: joi.string().allow(),
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, payment: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.uploadTaxInvoiceValidation = async (req, res, next) => {
    const data = req.body;
    if (data.voucher && typeof data.voucher == "string") data.voucher = JSON.parse(data.voucher);

    const schema = joi.object({
        invoiceId: joi.string(),
        invoicedAmount: joi.string(),
        invoiceNo: joi.string()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, invoice: data };
            next();
        }).catch(err => {
            console.log(err);
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(500).send({ status: false, error: errors });
        });
}

exports.mailSend = async (req, res, next) => {
    let data = req.body;
    if (data.otherInfo && typeof data.otherInfo == "string") data.otherInfo = JSON.parse(data.otherInfo);
    if (data.attachments && typeof data.attachments == "string") data.attachments = JSON.parse(data.attachments);

    const schema = joi.object({
        attachments: joi.array(),
        toEmails: joi.array(),
        ccEmails: joi.array(),
        bccEmails: joi.array(),
        subject: joi.string(),
        body: joi.string(),
        caseId: joi.string().allow('', null).optional(),
        mailType: joi.string(),
        invoice: joi.any(),
        otherInfo: joi.object()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, paymentSettlement: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.SavedReport = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        "title": joi.string().required(),
        "reportOf": joi.string().required(),
        clientType: joi.string().allow('', null).optional(),
        "client": joi.string().allow('', null).optional(),
        "branch": joi.string().allow('', null).optional(),
        "userId": joi.string().allow('', null).optional(),
        "organizationId": joi.string().allow('', null).optional(),
        "workType": joi.string().allow('', null).optional(),
        "start": joi.string().allow('', null).optional(),
        "end": joi.string().allow('', null).optional(),
        "caseStatus": joi.string().allow('', null).optional(),
        "dateRangeOn": joi.any().allow(null).optional()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, savedReport: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.createLeave = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        "reason": joi.string().required(),
        "dayType": joi.string().required(),
        "from": joi.string().required(),
        "to": joi.string().required()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, leave: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.updateLeave = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        "reason": joi.string().optional(),
        "dayType": joi.string().optional(),
        "from": joi.string().optional(),
        "to": joi.string().optional(),
        "leaveStatus": joi.string().optional()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, leave: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            res.status(409).send({ status: false, error: errors, body: req.body });
        });
}

exports.updateManualLog = async (req, res, next) => {
    let data = req.body;

    const schema = joi.object({
        "checkInTime": joi.date().optional(),
        "checkOutTime": joi.date().optional(),
        "date": joi.string().optional(),
        "logStatus": joi.string().optional()
    });

    schema.validateAsync(data, { abortEarly: false })
        .then(result => {
            req.local = { ...req.local, log: data };
            next();
        }).catch(err => {
            let errors = err.details.map(e => { return { key: e.path[0], message: e.message } })
            return res.status(409).send({ status: false, error: errors, body: req.body });
        });
}