const mongoose = require("mongoose");
const Schema = mongoose.Schema;
const moment = require('moment');
const { userSchema } = require('./CommonSchema');
const LeaveServices = require('../Services/CreateServices')("Leave");
const AttendanceServices = require('../Services/CreateServices')("Attendance");

const calendarSchema = Schema(
    {
        userId: {
            type: Schema.Types.ObjectId,
            ref: "User",
            required: true,
        },
        user: userSchema,
        orgId: {
            type: Schema.Types.ObjectId,
            ref: "Organization",
        },
        date: { type: Date, required: true },
        checkInTime: { type: Date },
        checkOutTime: { type: Date },
        workingHours: { type: Number, default: 0 },
        status: {
            type: String,
            enum: ["Present", "Absent", "Holiday", "Leave"],
            default: "Absent",
        },
        late: { type: Boolean, default: false },
        earlyLeave: { type: Boolean, default: false },
        overtime: { type: Boolean, default: false },
        breakDuration: { type: Boolean, default: false },
        note: { type: String },
        leaveStatus: { type: String, enum: ['PENDING', 'APPROVED', 'REJECTED'], default: 'PENDING' }, // Leave status for the day
    },
    { timestamps: true }
);


// Static method to reconcile attendance summaries
calendarSchema.statics.reconcileAttendanceSummary = async function (startDate, endDate) {
    try {
        // Fetch all attendance records for the date range
        const attendanceRecords = await AttendanceServices.Model.find({
            date: { $gte: startDate, $lte: endDate },
        }).populate('userId');

        // Fetch all leave records for the date range
        const leaveRecords = await LeaveServices.Model.find({
            from: { $lte: endDate }, // Leave starts on or before the end date
            to: { $gte: startDate }, // Leave ends on or after the start date
            leaveStatus: 'APPROVED', // Only consider approved leaves
        }).populate('userId');

        // Create a map of users on leave for each date
        const usersOnLeave = new Map();
        leaveRecords.forEach((leave) => {
            const leaveStart = moment(leave.from).startOf('day');
            const leaveEnd = moment(leave.to).endOf('day');
            const userId = leave.userId._id.toString();

            for (let date = leaveStart; date <= leaveEnd; date.add(1, 'day')) {
                const dateKey = date.format('YYYY-MM-DD');
                if (!usersOnLeave.has(dateKey)) {
                    usersOnLeave.set(dateKey, new Set());
                }
                usersOnLeave.get(dateKey).add(userId);
            }
        });

        // Group attendance records by user and date
        const userSummaries = new Map();
        attendanceRecords.forEach((record) => {
            const userId = record.userId._id.toString();
            const dateKey = moment(record.date).format('YYYY-MM-DD');

            if (!userSummaries.has(userId)) {
                userSummaries.set(userId, new Map());
            }

            const userDateSummaries = userSummaries.get(userId);
            if (!userDateSummaries.has(dateKey)) {
                userDateSummaries.set(dateKey, {
                    userId: record.userId._id,
                    orgId: record.orgId,
                    date: record.date,
                    checkInTime: record.checkIn.time,
                    checkOutTime: record.checkOut.time,
                    workingHours: record.workingHours || 0,
                    status: usersOnLeave.has(dateKey) && usersOnLeave.get(dateKey).has(userId) ? 'Leave' : 'Present',
                    late: false, // Add logic to calculate late
                    earlyLeave: false, // Add logic to calculate early leave
                    overtime: 0, // Add logic to calculate overtime
                    breakDuration: 0, // Add logic to calculate break duration
                    leaveStatus: usersOnLeave.has(dateKey) && usersOnLeave.get(dateKey).has(userId) ? 'APPROVED' : 'PENDING',
                });
            } else {
                const existingSummary = userDateSummaries.get(dateKey);

                // Update check-in time to the earliest
                if (record.checkIn.time < existingSummary.checkInTime) {
                    existingSummary.checkInTime = record.checkIn.time;
                }

                // Update check-out time to the latest
                if (record.checkOut.time > existingSummary.checkOutTime) {
                    existingSummary.checkOutTime = record.checkOut.time;
                }

                // Accumulate working hours
                existingSummary.workingHours += record.workingHours || 0;;
            }
        });

        // Handle users on leave but without attendance records
        leaveRecords.forEach((leave) => {
            const leaveStart = moment(leave.from).startOf('day');
            const leaveEnd = moment(leave.to).endOf('day');
            const userId = leave.userId._id.toString();

            for (let date = leaveStart; date <= leaveEnd; date.add(1, 'day')) {
                const dateKey = date.format('YYYY-MM-DD');
                if (!userSummaries.has(userId) || !userSummaries.get(userId).has(dateKey)) {
                    if (!userSummaries.has(userId)) {
                        userSummaries.set(userId, new Map());
                    }

                    userSummaries.get(userId).set(dateKey, {
                        userId: leave.userId._id,
                        orgId: leave.orgId,
                        date: date.toDate(),
                        checkInTime: null,
                        checkOutTime: null,
                        workingHours: 0,
                        status: 'Leave',
                        late: false,
                        earlyLeave: false,
                        overtime: 0,
                        breakDuration: 0,
                        leaveStatus: 'APPROVED',
                    });
                }
            }
        });

        // Save or update summaries in the AttendanceSummary collection
        for (const [userId, dateSummaries] of userSummaries) {
            for (const [dateKey, summary] of dateSummaries) {
                await this.findOneAndUpdate(
                    { userId: summary.userId, date: summary.date },
                    summary,
                    { upsert: true, new: true }
                );
            }
        }

        console.log('Attendance summaries reconciled successfully.');
    } catch (error) {
        console.error('Error reconciling attendance summaries:', error);
        throw error;
    }
};

module.exports = mongoose.model("AttendanceSummary", calendarSchema);