const Case = require('../Model/Case');

const defaultUserSelect = 'firstName lastName email profilePhoto';
const defaultPopulate = [
    { path: 'invoiceId' },
    { path: 'createdBy', select: defaultUserSelect },
    { path: 'outward.generatedBy', select: defaultUserSelect }
]

/**
 * @description This is used for get All cases
 * @param {*} query 
 * @param {*} displayFields 
 * @param {*} startIndex 
 * @param {*} limit 
 * @returns Object { status: true, cases: Array }
 *  { status: false, error: String }
 */
module.exports.getAllCases = async (query, { display = [], sort = { createdAt: -1 }, populate = defaultPopulate } = { display: [], sort: { createdAt: -1 }, populate: [] }, startIndex, limit) => {

    if (!query.type) {
        delete query.type
    } else {
        query.type = 'CASE';
    }

    return Case.find(query, display).populate([...defaultPopulate, ...populate]).skip(startIndex).sort(sort).limit(limit)
        .then((Cases) => {
            return { status: true, cases: Cases };
        }).catch((e) => {
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get list of lead and case 
 * @param {*} query 
 * @returns Object { status: true, cases: Array }
 *  { status: false, error: String }
 */
module.exports.getAllCaseLead = async (query) => {
    return Case.find(query).populate(defaultPopulate).sort({ createdAt: -1 }).then((Cases) => {
        return { status: true, cases: Cases };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, cases: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    return Case.countDocuments(query).then((Case) => {
        return { status: true, cases: Case };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for get One case 
 * @param {*} sparam 
 * @returns Object { status: true, cases: Object }
 *  { status: false, error: String }
 */
module.exports.getCase = async (sparam) => {
    let query = {};
    if (sparam.organization._id) {
        query['organization._id'] = sparam.organization._id;
    }
    if (sparam._id) { query._id = sparam._id; }

    return Case.findOne(query).populate(defaultPopulate).then((Case) => {
        return { status: true, cases: Case };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for get One case from an Id
 * @param {String} id 
 * @returns  Object { status: true, cases: Object }
 *  { status: false, error: String }
 */
module.exports.findCase = async (id) => {
    return Case.findById(id).populate(defaultPopulate).then((Case) => {
        return { status: true, cases: Case };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

module.exports.find = async (params = { query: {}, sort: { createdAt: -1 }, populate: [], start: 0, limit: 10, display: [], distinct: '_id' }) => {
    try {
        const {
            query = {},
            display = [],
            start = 0,
            limit = 10,
            populate = [],
            sort = '-createdAt',
            search = '',
            distinct = '_id'
        } = params;

        const count = await Case.countDocuments(query);
        const data = await Case.find(query, display)
            .populate([...defaultPopulate, ...populate])
            .sort(sort)
            .skip(start)
            .limit(limit)

        const totalPages = Math.ceil(count / limit);

        return {
            status: true,
            cases: data,
            count,
            totalPages,
        };
    }
    catch (e) {
        return { status: false, error: e.toString() };
    }
}

/**
 * @description Thisis used for create Case
 * @param {Object} caseData 
 * @returns Object { status: true, cases: Object }
 *  { status: false, error: String }
 */
module.exports.createCase = async (caseData) => {

    return Case.create(caseData)
        .then((result) => {
            return { status: true, cases: result };
        })
        .catch((err) => {
            return { status: false, error: err }
        })
};
/**
 * @description This is Used for update Case
 * @param {String} id 
 * @param {Object} caseData 
 * @returns Object { status: true, cases: Object }
 *  { status: false, error: String }
 */
module.exports.updateCase = async (id, caseData) => {
    try {
        const caseResult = await Case.findOneAndUpdate({ _id: id }, caseData, { new: true });
        if (!caseResult) return { status: false, error: "Case not found" };

        return { status: true, cases: caseResult };
    }
    catch (err) {
        return { status: false, error: err };
    }
};

/**
 * @description This is used for update Many cases
 * @param {Object} find_query 
 * @param {Object} case_data 
 * @returns Object { status: true, cases: result }
 * -  { status: false, err }
 */
module.exports.updateMultipleCase = async (find_query, case_data) => {
    return Case.updateMany(find_query, case_data, { new: true })
        .then((result) => {
            return { status: true, cases: result }
        })
        .catch((err) => {
            return { status: false, err }
        })
};

/**
 * @description This is used for update valuer in multiple cases 
 * @param {String} id 
 * @param {Object} update 
 * @param {Object} filter 
 * @returns Object {status : true, cases : result }
 * - {status : false, err}
 */
module.exports.updateValuerMultipleCase = async (id, update) => {
    return Case.updateMany(id, update)
        .then((result) => {
            return { status: true, cases: result }
        })
        .catch((err) => {
            return { status: false, err }
        })
};

/**
 * @description This is used for get All Completed Cases details  
 * @param {Object} query 
 * @param {String} orgId 
 * @returns Object { status: true, cases: Array }
 * -{ status: false, error: string }
 */
module.exports.nearByLocation = async (query) => {

    return Case.aggregate([
        {
            "$geoNear": {
                near: {
                    type: "Point",
                    coordinates: query.coordinates
                },
                maxDistance: query.distance,
                spherical: true,
                query: query.query,
                distanceField: "dist"
            }
        },
        {
            $project: {
                caseNumber: 1, "client.name": 1, siteMeasurement: { dateOfConstruction: { year: 1 }, noOfFloors: 1 }, "branch.name": 1, "location": 1, dist: 1, finalValuationAreaString: 1,
                geo: 1, address: 1, flatRate: 1, valuationDate: 1, reportGeneratedDate: 1, externalReportDetails: 1, outward: 1
            }
        }
    ])
        .then(result => {
            return { status: true, cases: result }
        }).catch(err => {
            return { status: false, error: err }
        })
};


/**
 * @description This is used for get All Completed Cases details  
 * @param {Object} query 
 * @param {String} orgId 
 * @returns Object { status: true, cases: Array }
 * -{ status: false, error: string }
 */
module.exports.aggregate = async (query) => {

    return Case.aggregate(query)
        .then(result => {
            return { status: true, data: result }
        }).catch(err => {
            return { status: false, error: err }
        })
};