const puppeteer = require('puppeteer');
const fs = require('fs');
const path = require('path');
const NodeGeocoder = require('node-geocoder');
const axios = require('axios');

const geocoder = NodeGeocoder({ 
    provider: 'openstreetmap',
    httpAdapter: 'https', 
    formatter: null,
    headers: { 'User-Agent': 'asdasd' } // Add a valid user-agent
});

async function getCoordinates(areaName, retries = 3) {
    for (let i = 0; i < retries; i++) {
        try {
            const res = await geocoder.geocode(areaName);
            if (res.length > 0) {
                return { lat: res[0].latitude, lon: res[0].longitude };
            }
            throw new Error('Location not found!');
        } catch (error) {
            console.error(`Attempt ${i + 1} failed: ${error.message}`);
            if (i === retries - 1) throw error;
            await new Promise(res => setTimeout(res, 2000)); // Wait before retrying
        }
    }
}

async function getGeoJSONBoundary(areaName) {
    const query = `
        [out:json];
        relation["name"="${areaName}"];
        (._;>;);
        out geom;
    `;
    const url = `https://overpass-api.de/api/interpreter?data=${encodeURIComponent(query)}`;
    try {
        const response = await axios.get(url);
        return response.data.elements.filter(el => el.type === 'way');
    } catch (error) {
        console.error('Error fetching GeoJSON:', error);
        return [];
    }
}
async function generateMap(lat, lon, zoom, filename, geojson) {
    const url = `https://www.openstreetmap.org/#map=${zoom}/${lat}/${lon}`;
    const browser = await puppeteer.launch({
        args: ['--no-sandbox', '--disable-setuid-sandbox']
    });
    const page = await browser.newPage();
    await page.setViewport({ width: 1920, height: 1080 });
    await page.goto(url, { waitUntil: 'networkidle2' });
    
    await new Promise(r => setTimeout(r, 3000));
    
    // Inject JavaScript to draw GeoJSON boundary
    await page.evaluate((geojson) => {
        const map = document.querySelector('#map');
        if (map && geojson.length > 0) {
            geojson.forEach(feature => {
                const coords = feature.geometry.coordinates.map(coord => [
                    (coord[0] - window.mapBounds[0]) / (window.mapBounds[2] - window.mapBounds[0]) * window.innerWidth,
                    (coord[1] - window.mapBounds[1]) / (window.mapBounds[3] - window.mapBounds[1]) * window.innerHeight
                ]);
                const polygon = document.createElement('div');
                polygon.style.position = 'absolute';
                polygon.style.border = '2px solid red';
                polygon.style.background = 'rgba(255, 0, 0, 0.2)';
                polygon.style.clipPath = `polygon(${coords.map(c => `${c[0]}px ${c[1]}px`).join(', ')})`;
                polygon.style.zIndex = '1000';
                map.appendChild(polygon);
            });
        }
    }, geojson);
    
    const screenshotsDir = path.join(__dirname, 'screenshots');
    if (!fs.existsSync(screenshotsDir)) {
        fs.mkdirSync(screenshotsDir);
    }
    
    const imagePath = path.join(screenshotsDir, `${filename}.png`);
    await page.screenshot({ path: imagePath });
    console.log(`Saved map with boundary: ${imagePath}`);
    
    await browser.close();
}

async function main(areaName) {
    try {
        const { lat, lon } = await getCoordinates(areaName);
        const geojson = await getGeoJSONBoundary(areaName);
        console.log("lngth: ",geojson.length);
        const zoomLevels = {
            '1000m_radius': 16,   // Close-up (1 km radius)
            '10km_radius': 14,    // City level (10 km radius)
            'district_level': 12, // District level
            'state_level': 10      // State level
        };
        
        for (const [name, zoom] of Object.entries(zoomLevels)) {
            await generateMap(lat, lon, zoom, `${areaName}_${name}`, geojson);
        }
        
        console.log('Maps captured and stored successfully!');
    } catch (error) {
        console.error(error);
    }
}

// Example usage:
const areaName = 'Andheri'; // Change this to your desired location
main(areaName);
